import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {localeMessages} from '../../services/locale-service';
import {XlReactWidgetReduxTable} from '../../xl-react-widget/xl-react-widget-redux-table/xl-react-widget-redux-table';
import DisplaySelectComponent from './bindings/display-select-component';
import ElementNavTitle from './element-nav-title';

import './redux-table-test-component.less';

export class ReduxTableTestComponent extends Component {
    static propTypes = {
        className: PropTypes.string,
        currentState: PropTypes.object.isRequired,
        fetchDataApi: PropTypes.func.isRequired,
        initializeReduxTable: PropTypes.func.isRequired,
        modelName: PropTypes.string.isRequired,
        showBindings: PropTypes.bool,
        tableId: PropTypes.string.isRequired,
        tableInitialized: PropTypes.bool.isRequired,
        updateState: PropTypes.func.isRequired
    };

    constructor(props) {
        super(props);
        const {currentState, modelName} = this.props;
        const model = currentState[modelName];
        this.state = {
            selectedRowData: model.selectedRowData,
            shouldShowFilters: model.shouldShowFilters
        };
    }

    componentDidMount() {
        this.props.initializeReduxTable();
    }

    handleRowDoubleClick = (row) => {
        this.setState({selectedRowData: row});
    };

    handleFilterTypeChange = (filterType) => {
        const {modelName, updateState, currentState} = this.props;
        updateState(modelName, {
            ...currentState[modelName],
            filterType
        });
    };

    selectRow = {
        classes: 'row-selected',
        clickToSelect: true,
        hideSelectColumn: true,
        mode: 'radio'
    };

    buildClassName = (dataKey) => `xl-react-table-${dataKey}`;

    createColumnDefinition = (headerDef) => {
        const {currentState, modelName} = this.props;
        const {fieldFormatters} = currentState[modelName];
        const dataFormat = R.propOr(R.identity, headerDef.dataKey, fieldFormatters);

        return {
            classes: this.buildClassName(headerDef.dataKey),
            dataField: headerDef.dataKey,
            formatter: dataFormat,
            headerClasses: this.buildClassName(headerDef.dataKey),
            headerTitle: false,
            sort: true,
            text: headerDef.title
        };
    };

    render() {
        const {
            className,
            currentState,
            fetchDataApi,
            modelName,
            showBindings,
            tableId,
            tableInitialized
        } = this.props;

        const model = currentState[modelName];
        const {
            fieldFormatters,
            filterType,
            keyField,
            loadingMessage,
            navigation: {
                description,
                title,
                type
            }
        } = model;

        const {filterProps} = model;

        const displayBindings = () =>
            showBindings ?
                <div className="testBindings">
                    <div>
                        <DisplaySelectComponent
                            clazzName={'type-of-filter'}
                            currentValue={filterType}
                            label={'Type of filter'}
                            listOfValues={['above', 'column']}
                            name={'type-of-filter'}
                            onSelectChange={this.handleFilterTypeChange}
                        />
                    </div>
                </div> :
                null;

        return (
            <div>
                <ElementNavTitle
                    description={description}
                    modelName={modelName}
                    title={title}
                    type={type}
                />
                <div className="testComponent">
                    {tableInitialized && <div className="xl-react-widget-redux-table">
                        <XlReactWidgetReduxTable
                            autoCleanup={false}
                            className={className}
                            createColumnDefinition={this.createColumnDefinition}
                            defaultErrorMessage={localeMessages.tableErrorMessage}
                            defaultNoDataText={localeMessages.tableEmptyDataMessage}
                            displayOnlyLabelIfNoData={false}
                            fetchDataApi={fetchDataApi}
                            fieldFormatters={fieldFormatters}
                            filterComponentProps={filterProps}
                            filterType={filterType}
                            keyField={keyField}
                            loadingMessage={loadingMessage}
                            onRowDoubleClick={this.handleRowDoubleClick}
                            register={this.registerTableApi}
                            selectRow={this.selectRow}
                            tableId={tableId}
                        />
                        {displayBindings()}
                    </div>}
                </div>
            </div>
        );
    }
}
