import R from 'ramda';
import {all, put, select, takeEvery} from 'redux-saga/effects';
import xlReactDipReduxFormActions from '../../xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-actions';
import xlReactWidgetReduxTableActions
    from '../../xl-react-widget/xl-react-widget-redux-table/xl-react-widget-redux-table-actions';
import kitchenSinkActions from '../actions/kitchen-sink-actions';
import {getFormById} from '../../xl-react-form/xl-react-dip-redux-form/xl-react-dip-form-selectors';
import {formInitialState} from '../form-page/form-initial-state';
import {
    fetchDataApi,
    tableWithDatePickerResource,
    tableWithToggleFilterResource
} from '../resource/redux-table-resource';
import {widgetInitialState} from '../widget-page/widget-initial-state';
import {localeMessages} from '../../services/locale-service';
import confirmationDialogActions
    from '../../xl-react-widget/xl-react-widget-confirmation-dialog-container/redux/confirmation-dialog-actions';

const createInitiaizeEffects = (...tableStates) =>
    tableStates.map(([{filters, layout, paging, sorting, tableId}, resourceMethod]) =>
        put(xlReactWidgetReduxTableActions.initialize({
            api: resourceMethod,
            filters,
            layout,
            loading: false,
            paging,
            sorting,
            tableId
        })));

function* initializeReduxTable() {
    const {
        widgetReduxTable,
        widgetReduxTable2,
        widgetReduxTable3,
        widgetReduxTable4,
        widgetReduxTable5
    } = widgetInitialState;

    const effects = createInitiaizeEffects(
        [widgetReduxTable, fetchDataApi],
        [widgetReduxTable2, fetchDataApi],
        [widgetReduxTable3, tableWithDatePickerResource],
        [widgetReduxTable4, tableWithToggleFilterResource],
        [widgetReduxTable5, fetchDataApi]
    );

    yield all(effects);
}

function* initializeReduxForm() {
    const keys = R.keys(formInitialState);
    for (let ind = 0; ind < keys.length; ind += 1) {
        const {
            formId,
            metadata,
            model,
            objLabel,
            validators
        } = formInitialState[keys[ind]];
        yield put(
            xlReactDipReduxFormActions.initialize({
                collapse: false,
                formId,
                metadata,
                model,
                objLabel,
                validators
            })
        );
    }
}

function* askThenClose() {
    yield put(confirmationDialogActions.showConfirmReject({
        confirm: localeMessages.leaveAction,
        dialogClassName: 'with-blue-header',
        message: localeMessages.unsavedChangesNotification,
        reject: localeMessages.stayOnPageButton,
        title: localeMessages.confirmLeaveSaveTitle
    }));
}

function* toggleMessageVisibility({modelName}) {
    const {message: {show}} = yield select(getFormById, modelName);
    yield put(xlReactDipReduxFormActions.mergeContext(modelName, {message: {show: !show}}));
}

export function* kitchenSinkSaga() {
    yield all([
        takeEvery(kitchenSinkActions.INITIALIZE_REDUX_FORM, initializeReduxForm),
        takeEvery(kitchenSinkActions.INITIALIZE_REDUX_TABLE, initializeReduxTable),
        takeEvery(kitchenSinkActions.OPEN_CONFIRMATION_DIALOG, askThenClose),
        takeEvery(kitchenSinkActions.TOGGLE_MESSAGE_VISIBILITY, toggleMessageVisibility)
    ]);
}

