import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import $ from 'jquery';
import R from 'ramda';
import moment from 'moment';

import classNames from 'classnames';

import JSONPretty from 'react-json-pretty';
import {PRINT_FORMAT, printValue} from '../../utils/components-utils';

export default class DisplayComponent extends PureComponent {
    static propTypes = {
        className: PropTypes.string,
        label: PropTypes.string,
        value: PropTypes.any
    };

    static hasToBePrettify(inputData) {
        if (Array.isArray(inputData) && !R.isEmpty(inputData)) {
            return $.isPlainObject(R.head(inputData));
        }
        return $.isPlainObject(inputData);
    }

    componentClass = classNames(this.props.className, 'value');

    modifyJsonForPrint(inputData) {
        return R.map((value) => {
            if (moment.isMoment(value)) {
                return value.format(PRINT_FORMAT);
            } else if (R.is(Date, value)) {
                return moment(value).format(PRINT_FORMAT);
            }
            return value;
        })(inputData);
    }

    prettifyJson(inputData) {
        const dataForPrint = this.modifyJsonForPrint(inputData);
        return <div className={this.componentClass}><JSONPretty json={dataForPrint}/></div>;
    }

    printAsString(inputData) {
        return <span className={this.componentClass}>{printValue(inputData)}</span>;
    }

    printData(inputData) {
        return DisplayComponent.hasToBePrettify(inputData) ?
            this.prettifyJson(inputData) : this.printAsString(inputData);
    }

    render() {
        const {label, value} = this.props;

        return <div className="displayTextBlock">
            <div className="label">{label}</div>
            {this.printData(value)}
        </div>;
    }
}
