/* eslint-disable sort-keys, object-curly-newline, no-multi-str, max-lines */
import React from 'react';
import R from 'ramda';
import moment from 'moment';
import * as KeyCodes from 'keycode-js';
import variables from '../../../app/styles/variables';
import {localeMessages} from '../../services/locale-service';
import {
    xlReactWidgetTableFilterStyle,
    xlReactWidgetTableFilterTypes
} from '../../xl-react-widget/xl-react-widget-redux-table/xl-react-widget-table-filter-type';
import {generateTree} from '../playground/playground-utils';
import RU from '../../utils/ramda-utils';
import hostname from '../../../gulp/utils/hostname';
import {widgetStringViews} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string-enum';
import {tagsAutocompleteSkins} from '../../xl-react-widget/xl-react-widget-tags/xl-react-widget-tags-skins';
import {genericDateFormat} from '../../utils/date-utils';
import {SIDES} from '../../xl-react-widget/xl-react-widget-split-panel/xl-react-widget-collapsible-vertical-split-pane/xl-react-widget-collapsible-vertical-split-pane-constants'; // eslint-disable-line max-len
import {
    actionBinding,
    apiBinding,
    booleanAnimateBinding,
    booleanAllowResizeBinding,
    booleanDisabledBinding,
    booleanOnlyUniqueSuggestionBinding,
    booleanReadOnlyBinding,
    booleanShowModalBinding,
    booleanStrippedBinding,
    commonBindings,
    displayFileTypeBinding,
    displayLastPushedPatchBinding,
    displayModelBinding,
    displayPristineBinding,
    displaySelectedActionBinding,
    displaySelectedLinesBinding,
    displayValidBinding,
    displayValueBinding,
    enumConfirmationDialogSkinBinding,
    enumDisplayModeBinding,
    enumFileTypeBinding,
    enumInputTypeBinding,
    enumProgressTypeBinding,
    enumSkinBinding,
    inputChangeViewBinding,
    inputDefaultValueBinding,
    inputSizeBinding,
    integerCurrentProgressBinding,
    maximumDateBinding,
    minimumDateBinding
} from '../playground/bindings/initial-bindings';
import kitchenSinkActions from '../actions/kitchen-sink-actions';
import jsonSample from './sample.json';
import yamlSample from './sample.yaml';

const type = 'widget';

const widgetDate1MinimumDate = moment({year: 2016, month: 10, day: 2});
const widgetDate1MaximumDate = moment({year: 2016, month: 10, day: 20});
const widgetDate1CurrentDate = moment({year: 2016, month: 10, day: 9});
const widgetTime1CurrentTime = moment({
    year: 2016, month: 10, day: 9, hours: 15, minutes: 10, seconds: 0, milliseconds: 0
});

export const widgetReduxTableData = [{
    firstname: 'John A. Smith',
    address: '1236 Some Street',
    city: 'San Francisco',
    state: 'CA'
}, {
    firstname: 'Joan B. Jones',
    address: '3271 Another Ave',
    city: 'New York',
    state: 'NY'
}, {
    firstname: 'Bob C. Uncle',
    address: '9996 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'John D. Smith',
    address: '1623 Some Street',
    city: 'San Francisco',
    state: 'CA'
}, {
    firstname: 'Bob F. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob G. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob H. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob I. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'Bob J. Uncle',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}, {
    firstname: 'John Doe',
    address: '9889 Random Road',
    city: 'Los Angeles',
    state: 'CA'
}];

export const autocompleteElements = R.map((ind) => ({
    id: `Application${ind}`,
    type: ind
}), R.range(1, 7));

const bindings = commonBindings('edit');

const stringBindings = (changeView = false) => [
    ...bindings,
    enumInputTypeBinding,
    inputSizeBinding,
    ...changeView ? [inputChangeViewBinding(widgetStringViews)] : [],
    inputDefaultValueBinding
];

export const widgetInitialState = {
    widgetAlert1: {
        alertType: 'success',
        navigation: {
            description: 'xl-react-widget-alert-message',
            title: 'Alerts widget',
            type
        }
    },
    widgetCategory: {
        open: true,
        navigation: {
            description: 'xl-react-widget-category',
            title: 'Collapsible panel',
            type
        }
    },
    widgetRadioGroup1: {
        bindings: [
            displayModelBinding,
            displayPristineBinding(),
            booleanDisabledBinding
        ],
        componentProps: {
            model: [{
                color: 'blue',
                label: 'Case 1',
                value: 'Case 1'
            }, {
                checked: true,
                color: 'yellow',
                label: 'Case 2',
                value: 'Case 2'
            }, {
                color: 'red',
                label: 'Case 3',
                value: 'Case 3'
            }]
        },
        navigation: {
            description: 'xl-react-widget-radio',
            title: 'Radio Widget',
            type
        }
    },
    widgetString1: {
        bindings: stringBindings(true),
        componentProps: {
            metadata: {
                default: '',
                description: 'This is my string field.',
                label: 'String me',
                placeholder: 'String placeholder',
                type: 'text'
            },
            model: ''
        },
        navigation: {
            description: 'xl-react-widget-string',
            title: 'String Widget',
            type
        }
    },
    widgetString2: {
        bindings: stringBindings(),
        componentProps: {
            metadata: {
                default: '',
                description: 'This is my string field.',
                label: 'String with addon',
                placeholder: 'String placeholder',
                type: 'text'
            },
            model: ''
        },
        navigation: {
            description: 'xl-react-widget-string-with-addon',
            title: 'String widget with addon',
            type
        }
    },
    widgetStringWithButtonAddon: {
        bindings,
        componentProps: {
            metadata: {
                default: '',
                description: 'This is my string field.',
                label: 'String with button addon',
                placeholder: 'String placeholder'
            },
            model: {
                button: '',
                filter: ''
            }
        },
        navigation: {
            description: 'xl-react-widget-string-with-button-addon',
            title: 'String widget with button addon',
            type
        }
    },
    widgetPatchSample: {
        bindings: [displaySelectedLinesBinding],
        componentProps: {
            filePath: 'Environments/Patch-1',
            headerLabel: 'Sample',
            model: yamlSample,
            selectedLines: []
        },
        navigation: {
            description: 'xl-react-widget-patch-sample',
            title: 'Patch sample',
            type
        }
    },
    widgetPatchEditor: {
        bindings: [
            displayPristineBinding(),
            displaySelectedLinesBinding,
            displaySelectedActionBinding,
            displayFileTypeBinding,
            displayLastPushedPatchBinding,
            enumFileTypeBinding
        ],
        componentProps: {
            editableSection: '',
            filePath: 'Environments/Patch-1',
            fileType: 'yaml',
            initialYamlModel: yamlSample,
            initialJsonModel: jsonSample,
            lastPushedPatch: '',
            model: yamlSample,
            pristine: true,
            selectedAction: 'none',
            selectedLines: [],
            selectedPath: ''
        },
        navigation: {
            description: 'xl-react-widget-patch-editor',
            title: 'Patch editor',
            type
        }
    },
    widgetProgress1: {
        bindings: [
            displayModelBinding,
            displayPristineBinding(),
            displayValidBinding(),
            enumDisplayModeBinding('edit'),
            apiBinding,
            integerCurrentProgressBinding,
            booleanStrippedBinding,
            enumProgressTypeBinding
        ],
        componentProps: {
            metadata: {
                description: 'This is my string field.',
                label: 'I\'m required string',
                placeholder: 'String placeholder',
                required: true
            },
            model: 150,
            minValue: 0,
            maxValue: 200,
            pristine: true,
            valid: true
        },
        navigation: {
            description: 'xl-react-widget-progressbar',
            title: 'Progress bar',
            type
        }
    },
    widgetFilter1: {
        componentProps: {
            metadata: {
                label: 'I\'m a filter',
                placeholder: 'I\'m a placeholder'
            },
            model: ''
        },
        navigation: {
            description: 'xl-react-widget-filter',
            title: 'Filter widget',
            type
        }
    },
    widgetTags1: {
        bindings: commonBindings('edit', true),
        componentProps: {
            metadata: {
                description: 'Click me.',
                default: R.map(
                    (value) => ({label: value, value}),
                    ['test']
                ),
                label: 'insert a tag',
                name: 'tags.test',
                required: true
            },
            model: []
        },
        navigation: {
            description: 'xl-react-widget-tags',
            title: 'Tags Widget',
            type
        }
    },
    widgetTags2: {
        bindings: commonBindings('edit', true),
        componentProps: {
            metadata: {
                description: 'Click me.',
                default: [],
                label: 'insert a tag',
                name: 'tags.test',
                required: true,
                requiredText: 'Custom required text overriding default required text!'
            },
            model: []
        },
        navigation: {
            description: 'xl-react-widget-tags',
            title: 'Tags Widget (Custom default error)',
            type
        }
    },
    widgetTags3: {
        bindings: [
            ...bindings,
            enumSkinBinding,
            booleanOnlyUniqueSuggestionBinding
        ],
        componentProps: {
            metadata: {
                label: 'Tags with autocomplete',
                name: 'widgetTags3',
                description: 'These are autocomplete tags.',
                placeholder: 'placeholder'
            },
            model: [{
                color: variables['xl-green'],
                label: 'Application 0',
                value: 'Applications/TestApp/Application 0'
            }]
        },
        navigation: {
            description: 'xl-react-widget-tags',
            title: 'Tags Widget (Autocomplete input)',
            type
        }
    },
    widgetDate1: {
        bindings: [
            ...bindings,
            minimumDateBinding(widgetDate1MinimumDate),
            maximumDateBinding(widgetDate1MaximumDate)
        ],
        componentProps: {
            minDate: widgetDate1MinimumDate,
            maxDate: widgetDate1MaximumDate,
            model: widgetDate1CurrentDate,
            metadata: {
                description: 'Date picker description',
                name: 'my date',
                label: 'Choose a date',
                format: genericDateFormat
            }
        },
        navigation: {
            description: 'xl-react-widget-date',
            title: 'Date picker Widget',
            type
        }
    },
    widgetTime1: {
        bindings,
        componentProps: {
            metadata: {
                label: 'Choose time',
                description: 'Widget time description'
            },
            model: widgetTime1CurrentTime
        },
        navigation: {
            description: 'xl-react-widget-time',
            title: 'Time picker Widget',
            type
        }
    },
    widgetAutocomplete1: {
        bindings: commonBindings('edit', true),
        componentProps: {
            metadata: {
                label: 'Autocomplete',
                name: 'application.autocomplete',
                description: 'This is my autocomplete field.',
                placeholder: 'autocomplete placeholder',
                required: true,
                requiredText: 'Custom required error message overrides!'
            }
        },
        navigation: {
            description: 'xl-react-widget-autocomplete',
            title: 'Autocomplete Widget',
            type
        }
    },
    widgetAutocomplete2: {
        bindings: commonBindings('edit', true),
        componentProps: {
            metadata: {
                default: 'Application2',
                label: 'Autocomplete',
                name: 'application.autocomplete',
                description: 'This is my autocomplete field.',
                placeholder: 'autocomplete placeholder',
                required: true,
                requiredText: 'Custom required error message overrides!'
            }
        },
        navigation: {
            description: 'xl-react-widget-autocomplete',
            title: 'Autocomplete Widget (Default value)',
            type
        }
    },
    widgetAutocomplete3: {
        bindings: commonBindings('edit', true),
        componentProps: {
            metadata: {
                default: {id: 'Application2', type: 2},
                label: 'Autocomplete',
                name: 'application.autocomplete',
                description: 'This is my autocomplete field.',
                placeholder: 'autocomplete placeholder',
                required: true,
                requiredText: 'Custom required error message overrides!'
            }
        },
        navigation: {
            description: 'xl-react-widget-autocomplete',
            title: 'Autocomplete Widget Model Objects (Default value)',
            type
        }
    },
    widgetAutocomplete4: {
        bindings: commonBindings('edit', true),
        componentProps: {
            metadata: {
                label: 'Autocomplete',
                name: 'application.autocomplete',
                description: 'This is my autocomplete field.',
                placeholder: 'autocomplete placeholder',
                required: true,
                requiredText: 'Custom required error message overrides!'
            }
        },
        navigation: {
            description: 'xl-react-widget-autocomplete-with-addons',
            title: 'Autocomplete Widget with addons',
            type
        }
    },
    widgetAutocomplete5: {
        bindings: commonBindings('edit', true),
        componentProps: {
            metadata: {
                label: 'Autocomplete',
                name: 'application.autocomplete.delayed',
                description: 'This is my autocomplete field.',
                placeholder: 'autocomplete placeholder',
                required: true,
                requiredText: 'Custom required error message overrides!'
            }
        },
        navigation: {
            description: 'xl-react-widget-autocomplete-with-delay',
            title: 'Autocomplete Widget with delay (spinner)',
            type
        }
    },
    widgetFileUpload1: {
        bindings,
        componentProps: {
            metadata: {
                auto: false,
                browseButtonType: 'secondary',
                label: 'Upload me',
                description: 'File Upload description',
                uploadUrl: `http://${hostname}:3002/upload`
            }
        },
        navigation: {
            description: 'xl-react-widget-file-upload',
            title: 'File upload Widget',
            type
        }
    },
    widgetModalDialog1: {
        bindings: [
            booleanAnimateBinding,
            booleanShowModalBinding('showModal')
        ],
        componentProps: {
            showModal: false,
            title: 'Modal dialog'
        },
        navigation: {
            description: 'xl-react-widget-modal-dialog',
            title: 'Modal Dialog Widget',
            type
        },
        text: 'Lorem Ipsum is simply dummy text of the printing and typesetting industry. \
            Lorem Ipsum has been the industry\'s standard dummy text ever since the 1500s, when \
            an unknown printer took a galley of type and scrambled it to make a type specimen book. \
            It has survived not only five centuries, but also the leap into electronic typesetting, \
            remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset \
            sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like \
            Aldus PageMaker including versions of Lorem Ipsum.'
    },
    widgetModalDialog2: {
        bindings: [
            booleanAnimateBinding,
            booleanShowModalBinding('showModal')
        ],
        componentProps: {
            expandable: true,
            showModal: false,
            title: 'Expandable modal dialog'
        },
        navigation: {
            description: 'xl-react-widget-modal-dialog',
            title: 'Expandable modal dialog',
            type
        },
        text: 'Lorem Ipsum is simply dummy text of the printing and typesetting industry. \
            Lorem Ipsum has been the industry\'s standard dummy text ever since the 1500s, when \
            an unknown printer took a galley of type and scrambled it to make a type specimen book. \
            It has survived not only five centuries, but also the leap into electronic typesetting, \
            remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset \
            sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like \
            Aldus PageMaker including versions of Lorem Ipsum.'
    },
    widgetConfirmationDialog1: {
        bindings: [
            booleanShowModalBinding('show'),
            booleanAnimateBinding,
            enumConfirmationDialogSkinBinding
        ],
        componentProps: {
            buttons: [
                {
                    action: 'confirm',
                    className: 'xl-primary',
                    label: 'Yes',
                    keyCodes: [KeyCodes.KEY_RETURN]
                },
                {
                    action: 'reject',
                    className: 'xl-secondary',
                    label: 'No',
                    keyCodes: [KeyCodes.KEY_ESCAPE]
                },
                {
                    action: 'noop',
                    className: '',
                    disabled: true,
                    label: 'Disabled button',
                    keyCodes: [KeyCodes.KEY_0]
                }
            ],
            title: 'Confirmation dialog with a long title name which doesn\'t fit the default size of modal window'
        },
        navigation: {
            description: 'xl-react-widget-confirmation-dialog',
            title: 'Confirmation Dialog Widget',
            type
        },
        text: 'Are you sure you want to leave? Your changes will be lost.'
    },
    widgetConfirmationDialogContainer: {
        bindings: [
            actionBinding('Show', {type: kitchenSinkActions.OPEN_CONFIRMATION_DIALOG})
        ],
        componentProps: {},
        navigation: {
            description: 'xl-react-widget-confirmation-dialog-container',
            title: 'Confirmation Dialog Container Widget',
            type
        }
    },
    widgetTree1: {
        autoOpen: false,
        className: 'xl-react-widget-tree',
        model: [
            {
                id: 'node--1',
                children: generateTree(2, 3),
                name: 'Pre-Loaded'
            },
            {
                id: 'node--2',
                name: 'Empty-Node'
            }
        ],
        selectedNodes: [],
        selectionMode: 'multi',
        selectable: true,
        navigation: {
            description: 'xl-react-widget-tree',
            title: 'Tree Widget',
            type
        }
    },
    widgetTree2: {
        autoOpen: false,
        className: 'xl-react-widget-tree-lazy',
        model: [
            {
                id: 'node--1',
                loadOnDemand: true,
                name: 'Lazy-Loaded'
            }
        ],
        selectedNodes: [],
        selectionMode: 'multi',
        selectable: true,
        navigation: {
            description: 'xl-react-widget-tree',
            title: 'Lazy Tree Widget',
            type
        }
    },
    widgetTreeLazyLoaded: {
        className: 'xl-react-widget-tree-lazy-loaded',
        model: [
            {
                id: 'node--1',
                name: 'Lazy-Loaded Tree 1'
            },
            {
                id: 'node--2',
                name: 'Lazy-Loaded Tree 2'
            },
            {
                id: 'node--3',
                name: 'Lazy-Loaded Tree 3'
            },
            {
                id: 'node--4',
                name: 'Lazy-Loaded Tree 4'
            },
            {
                id: 'node--5',
                name: 'Lazy-Loaded Tree 5'
            },
            {
                id: 'node--6',
                name: 'Lazy-Loaded Tree 6'
            },
            {
                id: 'node--7',
                name: 'Lazy-Loaded Tree 7'
            },
            {
                id: 'node--8',
                name: 'Lazy-Loaded Tree 8'
            },
            {
                id: 'node--9',
                name: 'Lazy-Loaded Tree 9'
            },
            {
                id: 'node--10',
                name: 'Lazy-Loaded Tree 10'
            }
        ],
        navigation: {
            description: 'xl-react-widget-tree-lazy-loaded',
            title: 'Lazy Loaded Tree Widget',
            type
        }
    },
    widgetBottomSplitPane: {
        componentProps: {},
        allowResize: true,
        defaultPaneHeight: '80%',
        minSizeBottom: 27,
        minSizeTop: 35,
        showBindings: false,
        navigation: {
            description: 'xl-react-widget-bottom-split-pane',
            title: 'Horizontal split pane Widget',
            type
        }
    },
    widgetVerticalSplitPane: {
        componentProps: {},
        bindings: [
            booleanAllowResizeBinding
        ],
        allowResize: true,
        isCollapsed: false,
        minPrimarySideWidth: 100,
        minSecondarySideWidth: 100,
        primary: SIDES.primary,
        primarySideWidth: 100,
        shouldAnimate: true,
        sideToCollapse: SIDES.primary,
        navigation: {
            description: 'xl-react-widget-collapsible-vertical-split-pane',
            title: 'Vertical split pane Widget',
            type
        }
    },
    widgetTable: {
        data: [{
            firstname: 'John A. Smith',
            address: '1236 Some Street',
            city: 'San Francisco',
            state: 'CA'
        }, {
            firstname: 'Joan B. Jones',
            address: '3271 Another Ave',
            city: 'New York',
            state: 'NY'
        }, {
            firstname: 'Bob C. Uncle',
            address: '9996 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'John D. Smith',
            address: '1623 Some Street',
            city: 'San Francisco',
            state: 'CA'
        }, {
            firstname: 'Bob F. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob G. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob H. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob I. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'Bob J. Uncle',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }, {
            firstname: 'John Doe',
            address: '9889 Random Road',
            city: 'Los Angeles',
            state: 'CA'
        }],
        fieldFormatters: {
            completionDate: (arg) => arg,
            startDate: (arg) => arg,
            status: (arg) => arg,
            type: (arg) => arg
        },
        filters: {
            firstname: '',
            address: '',
            city: '',
            state: ''
        },
        keyField: 'firstname',
        layout: RU.mapIndexed(([key, title], id) => ({
            dataKey: key,
            sortKey: key,
            title: title,
            id
        }), [['firstname', 'Firstname'], ['address', 'Address'], ['city', 'City'], ['state', 'State']]),
        loading: false,
        loadingMessage: localeMessages.tableLoadingMessage,
        paging: {
            page: 1,
            pageSizes: [{
                text: '2',
                value: 2
            }, {
                text: '5',
                value: 5
            }, {
                text: '10',
                value: 10
            }],
            pageStartIndex: 1,
            resultsPerPage: 2,
            prePageText: '<',
            nextPageText: '>',
            paginationPosition: 'bottom'
        },
        selectedRowData: {},
        shouldShowFilters: false,
        sorting: {name: 'startDate', order: 'desc'},
        total: 10,
        navigation: {
            description: 'xl-react-widget-table',
            title: 'Table Widget',
            type
        }
    },
    widgetReduxTable: {
        columnsFilter: {
            firstname: '',
            address: '',
            city: '',
            state: ''
        },
        data: R.slice(0, 2, widgetReduxTableData),
        fieldFormatters: {},
        filters: [
            {
                className: '',
                fieldPath: 'firstname-filter-key',
                label: 'First name',
                key: 'firstname',
                model: '',
                placeholder: 'Filter by first name',
                position: {
                    column: 0,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: 'address',
                label: 'Address',
                key: 'address',
                model: '',
                placeholder: 'Filter by address',
                position: {
                    column: 0,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: () => 'city',
                label: 'City',
                key: 'city',
                model: '',
                placeholder: 'Filter by city',
                position: {
                    column: 1,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: 'state',
                label: 'State',
                key: 'state',
                model: '',
                placeholder: 'Filter by state',
                position: {
                    column: 1,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            }
        ],
        filterType: 'above',
        keyField: 'firstname',
        layout: RU.mapIndexed(
            ([key, title, sortKey], id) => ({
                dataKey: key,
                sortKey,
                title: title,
                id
            }),
            [
                ['firstname', 'Firstname', 'firstname'],
                ['address', 'Address', 'address-sort-key'],
                ['city', 'City', 'city'],
                ['state', 'State', 'state']
            ]
        ),
        loading: false,
        navigation: {
            description: 'xl-react-widget-redux-table',
            title: 'Redux Table Widget',
            type
        },
        paging: {
            page: 1,
            pageSizes: [
                {
                    text: '2',
                    value: 2
                },
                {
                    text: '5',
                    value: 5
                },
                {
                    text: '10',
                    value: 10
                }
            ],
            resultsPerPage: 2
        },
        selectedRowData: {},
        shouldShowFilters: false,
        sorting: {name: 'firstname', order: 'desc'},
        tableId: 'redux-table-1',
        total: 10
    },
    widgetReduxTable2: {
        data: R.slice(0, 2, widgetReduxTableData),
        fieldFormatters: {},
        filters: [
            {
                className: '',
                fieldPath: 'firstname-filter-key',
                label: 'First name',
                key: 'firstname',
                model: '',
                placeholder: 'Filter by first name',
                position: {
                    column: 0,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: 'address',
                label: 'Address',
                key: 'address',
                model: '',
                placeholder: 'Filter by address',
                position: {
                    column: 0,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.INPUT
            },
            {
                className: '',
                fieldPath: () => 'city',
                label: 'City',
                key: 'city',
                model: '',
                placeholder: 'Filter by city',
                position: {
                    column: 1,
                    row: 0
                },
                type: xlReactWidgetTableFilterTypes.AUTOCOMPLETE
            },
            {
                className: '',
                fieldPath: 'state',
                label: 'State',
                key: 'state',
                model: [],
                placeholder: 'Filter by state',
                position: {
                    column: 1,
                    row: 1
                },
                type: xlReactWidgetTableFilterTypes.TAGS
            }
        ],
        filterType: xlReactWidgetTableFilterStyle.ABOVE,
        filterProps: {
            state: {
                handlers: {
                    autocomplete: {
                        addCandidates: () => Promise.resolve([
                            {value: 'CA'},
                            {value: 'NY'}
                        ])
                    }
                },
                objLabel: 'value',
                skin: tagsAutocompleteSkins.CHECKBOX
            },
            city: {
                handlers: {
                    addCandidates: () => Promise.resolve([
                        {id: 'Los Angeles'},
                        {id: 'New York'},
                        {id: 'San Francisco'}
                    ])
                }
            }
        },
        keyField: 'firstname',
        layout: RU.mapIndexed(
            ([key, title, sortKey], id) => ({
                dataKey: key,
                sortKey,
                title: title,
                id
            }),
            [
                ['firstname', 'Firstname', 'firstname'],
                ['address', 'Address', 'address-sort-key'],
                ['city', 'City', 'city'],
                ['state', 'State', 'state']
            ]
        ),
        loading: false,
        navigation: {
            description: 'xl-react-widget-redux-table-with-autocomplete-filter',
            title: 'Redux Table Widget with autocomplete filters',
            type
        },
        paging: {
            page: 1,
            pageSizes: [
                {
                    text: '2',
                    value: 2
                },
                {
                    text: '5',
                    value: 5
                },
                {
                    text: '10',
                    value: 10
                }
            ],
            resultsPerPage: 2
        },
        selectedRowData: {},
        shouldShowFilters: false,
        sorting: {name: 'firstname', order: 'desc'},
        tableId: 'redux-table-2',
        total: 10
    },
    widgetReduxTable3: {
        data: [{
            dateOfBirth: moment({year: 2016, month: 10, day: 2})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 3})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 4})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 5})
        }, {
            dateOfBirth: moment({year: 2016, month: 10, day: 6})
        }],
        fieldFormatters: {dateOfBirth: (thisMoment) => thisMoment.toString()},
        filters: [{
            className: '',
            fieldPath: 'afterDateOfBirth',
            label: 'Date Of Birth after',
            key: 'afterDateOfBirth',
            model: moment({year: 2016, month: 10, day: 3}),
            placeholder: 'Date Of Birth after',
            position: {
                column: 0,
                row: 0
            },
            type: xlReactWidgetTableFilterTypes.DATEPICKER
        }, {
            className: '',
            fieldPath: 'beforeDateOfBirth',
            label: 'Date Of Birth before',
            key: 'beforeDateOfBirth',
            model: moment({year: 2016, month: 10, day: 4}),
            placeholder: 'Date Of Birth before',
            position: {
                column: 1,
                row: 0
            },
            type: xlReactWidgetTableFilterTypes.DATEPICKER
        }],
        filterType: xlReactWidgetTableFilterStyle.ABOVE,
        keyField: 'dateOfBirth',
        layout: [{
            dataKey: 'dateOfBirth',
            dataSort: false,
            sortKey: 'dateOfBirth',
            title: 'Date Of Birth'
        }],
        loading: false,
        navigation: {
            description: 'xl-react-widget-redux-table-with-date-picker',
            title: 'Redux Table Widget with Date Picker',
            type
        },
        paging: {
            page: 1,
            pageSizes: [{
                text: '5',
                value: 5
            }],
            pageStartIndex: 1,
            resultsPerPage: 5
        },
        sorting: {name: 'dateOfBirth', order: 'asc'},
        tableId: 'redux-table-3',
        total: 5
    },
    widgetReduxTable4: {
        data: R.range(1, 101).map(R.objOf('value')),
        filters: [{
            className: '',
            fieldPath: 'value',
            label: 'Condition',
            key: 'value',
            model: 'all',
            position: {
                column: 0,
                row: 0
            },
            metadata: [
                {label: 'All', model: 'all'},
                {label: 'Value < 50', model: 'lt50'},
                {label: 'Value > 50', model: 'gt50'},
                {label: 'Value is greater than 50', model: 'gt50'}
            ],
            objLabel: 'label',
            objValue: 'model',
            type: xlReactWidgetTableFilterTypes.TOGGLE_BUTTONS
        }],
        filterType: xlReactWidgetTableFilterStyle.ABOVE,
        keyField: 'value',
        layout: [{
            dataKey: 'value',
            dataSort: false,
            sortKey: 'value',
            title: 'Value'
        }],
        loading: false,
        navigation: {
            description: 'xl-react-widget-redux-table-with-toggle-button-filter',
            title: 'Redux Table Widget with Toggle Button filter',
            type
        },
        paging: {
            page: 1,
            pageSizes: [{
                text: '5',
                value: 5
            }],
            pageStartIndex: 1,
            resultsPerPage: 5
        },
        sorting: {name: 'value', order: 'asc'},
        tableId: 'redux-table-4',
        total: 100
    },
    widgetReduxTable5: {
        data: R.slice(0, 2, widgetReduxTableData),
        fieldFormatters: {},
        filters: [],
        filterType: xlReactWidgetTableFilterStyle.ABOVE,
        filterProps: {
            state: {
                handlers: {
                    autocomplete: {
                        addCandidates: () => Promise.resolve([
                            {value: 'CA'},
                            {value: 'NY'}
                        ])
                    }
                },
                objLabel: 'value',
                skin: tagsAutocompleteSkins.CHECKBOX
            },
            city: {
                handlers: {
                    addCandidates: () => Promise.resolve([
                        {id: 'Los Angeles'},
                        {id: 'New York'},
                        {id: 'San Francisco'}
                    ])
                }
            }
        },
        keyField: 'firstname',
        layout: RU.mapIndexed(
            ([key, title, sortKey], id) => ({
                dataKey: key,
                sortKey,
                title: title,
                id
            }),
            [
                ['firstname', 'Firstname', 'firstname'],
                ['address', 'Address', 'address-sort-key'],
                ['city', 'City', 'city'],
                ['state', 'State', 'state']
            ]
        ),
        loading: false,
        navigation: {
            description: 'xl-react-widget-redux-table-with-autocomplete-filter',
            title: 'Redux Table Widget with autocomplete filters',
            type
        },
        paging: {
            page: 1,
            pageSizes: [
                {
                    text: '2',
                    value: 2
                },
                {
                    text: '5',
                    value: 5
                },
                {
                    text: '10',
                    value: 10
                }
            ],
            pageButtonRenderer: (pagination) => <h2>{pagination.page}</h2>,
            sizePerPageRenderer: () => <h3>Current size</h3>,
            resultsPerPage: 2
        },
        selectedRowData: {},
        shouldShowFilters: false,
        sorting: {name: 'firstname', order: 'desc'},
        tableId: 'redux-table-5',
        total: 10
    },
    widgetTreeSelect: {
        data: [
            {
                id: 'FOLDER_1',
                title: 'Folder 1',
                enabled: true
            },
            {
                children: [{
                    children: [{
                        children: [{
                            id: 'FOLDER_2111',
                            title: 'Folder 2111',
                            enabled: false
                        }, {
                            id: 'FOLDER_2112',
                            title: 'Folder 2112',
                            enabled: false
                        }],
                        id: 'FOLDER_211',
                        title: 'Folder 211',
                        enabled: false
                    }, {
                        id: 'FOLDER_212',
                        title: 'Folder 212',
                        enabled: false
                    }, {
                        id: 'FOLDER_213',
                        title: 'Folder 213',
                        enabled: false
                    }],
                    id: 'FOLDER_21',
                    title: 'Folder 21',
                    enabled: false
                }, {
                    children: [{
                        id: 'FOLDER_222',
                        title: 'Folder 222',
                        enabled: false
                    }],
                    id: 'FOLDER_22',
                    title: 'Folder 22',
                    enabled: true
                }],
                id: 'FOLDER_2',
                title: 'Folder 2',
                enabled: true
            },
            {
                id: 'FOLDER_3',
                title: 'Folder 3',
                enabled: false
            },
            {
                id: 'FOLDER_4',
                title: 'Folder 4',
                enabled: false
            },
            {
                id: 'FOLDER_5',
                title: 'Folder 5',
                enabled: false
            },
            {
                children: [{
                    children: [{
                        children: [{
                            id: 'FOLDER_6111',
                            title: 'Folder 6111',
                            enabled: false
                        }, {
                            id: 'FOLDER_6112',
                            title: 'Folder 6112',
                            enabled: false
                        }],
                        id: 'FOLDER_611',
                        title: 'Folder 611',
                        enabled: false
                    }],
                    id: 'FOLDER_61',
                    title: 'Folder 61',
                    enabled: false
                }],
                id: 'FOLDER_6',
                title: 'Folder 6',
                enabled: false
            }
        ],
        model: {
            id: 'FOLDER_6111',
            title: 'Folder 6111',
            enabled: true
        },
        navigation: {
            description: 'xl-react-widget-tree-select',
            title: 'Folder Tree Select Widget',
            type
        }
    },
    widgetSpinner: {
        navigation: {
            description: 'xl-react-widget-spinner',
            title: 'Spinner widget',
            type
        },
        overlay: true,
        message: localeMessages.spinnerMessage,
        show: true
    },
    widgetSwitch: {
        bindings,
        componentProps: {
            metadata: {label: 'Switch button'},
            model: true,
            pristine: true,
            valid: true
        },
        navigation: {
            description: 'xl-react-widget-switch',
            title: 'Widget switch',
            type
        }
    },
    widgetWithPopover: {
        navigation: {
            description: 'xl-react-widget-with-popover',
            title: 'Widget with Popover',
            type
        },
        popoverContent: 'Widget with popover content'
    },
    widgetMonacoDiffViewer: {
        navigation: {
            description: 'xl-react-widget-monaco-diff-viewer',
            title: 'Monaco diff viewer',
            type
        },
        value: 'Value',
        original: 'Original'
    },
    widgetMonacoEditor: {
        bindings: [
            displayValueBinding,
            booleanReadOnlyBinding
        ],
        componentProps: {
            displayedProperties: [{
                className: 'value',
                label: 'Value',
                name: 'value'
            }],
            options: {readOnly: true},
            value: ''
        },
        navigation: {
            description: 'xl-react-widget-monaco-editor',
            title: 'Monaco editor',
            type
        }
    },
    widgetLastTimeUpdated: {
        value: new Date(),
        navigation: {
            description: 'xl-react-widget-last-time-updated',
            title: 'Last time updated',
            type
        }
    }
};
