import R from 'ramda';
import {defaultValidatorForRequiredField} from '../validators/default-validator-for-required-field';
import {invokeCallback} from '../utils/function-utils';
import {validateComponent} from './component-validate-service';

/* eslint-disable complexity */
export const computeNextState = (context, props, validators, model, {
    allowUndefinedModel = false,
    invokeCallbackEnabled = true,
    customCallbacks = [],
    pristine = false
} = {}) => {
    let dataForCallbacks = [...customCallbacks];
    if (props.independent) {
        const {message, model: propsModel, pristine: propsPristine, valid} = props;
        const isNewModel = model !== propsModel && pristine === false;
        context.setState({message, model: isNewModel ? model : propsModel, pristine: propsPristine, valid});
        if (isNewModel) {
            props.onModelChange(model);
        }
        R.forEach((data) => invokeCallback(props, data), dataForCallbacks);
        return;
    }
    const prevState = context.state;
    const {message = props.message, valid} = validateComponent(props, validators, model, pristine);
    const newState = {};

    const isModelDefined = allowUndefinedModel || !R.isNil(model);

    if (prevState.valid !== valid) {
        newState.valid = valid;
        dataForCallbacks = [...dataForCallbacks, {functionName: 'onValidityChange', values: [valid]}];
    }

    if (!R.equals(prevState.message, message)) {
        newState.message = message;
        dataForCallbacks = [...dataForCallbacks, {
            functionName: 'onMessageChange',
            values: [message]
        }, {
            functionName: 'messageHandler',
            values: [message]
        }];
    }

    if (prevState.pristine !== pristine) {
        newState.pristine = pristine;
        dataForCallbacks = [...dataForCallbacks, {functionName: 'onPristinityChange', values: [pristine]}];
    }

    if (isModelDefined && !R.equals(prevState.model, model)) {
        newState.model = model;
        dataForCallbacks = [...dataForCallbacks, {
            functionName: 'onModelChange',
            values: [model, pristine]
        }];
    }

    context.setState(newState, () => {
        if (invokeCallbackEnabled) {
            R.forEach((data) => invokeCallback(props, data), dataForCallbacks);
        }
    });
};

export const computeOnChangeState = (context, props, validators, model, config = {}) =>
    computeNextState(context, props, validators, model, {...config, allowUndefinedModel: true});

export const computeNextPropsState = (context, props, state, validators, invokeCallbackEnabled = true) => {
    const model = !R.isNil(props.model) ? props.model : state.model;
    if (!R.isNil(model)) {
        computeNextState(context, props, validators, model, {
            invokeCallbackEnabled,
            pristine: state.pristine
        });
    }
};

export const computeInitialState = (context, props, validators, model, invokeCallbackEnabled = true) => {
    if (!R.isNil(model)) {
        computeNextState(context, props, validators, model, {
            customCallbacks: context.api ? [{functionName: 'register', values: [context.api]}] : [],
            invokeCallbackEnabled,
            pristine: true
        });
    }
};

export const getDefaultValidatorForRequiredField = (props) => {
    const config = props.metadata ? props.metadata : props.options;
    return defaultValidatorForRequiredField(
        Boolean(config.required || config.asRequired),
        config.requiredText
    );
};

export const getValidators = (props, customValidators = []) =>
    [
        getDefaultValidatorForRequiredField(props),
        ...customValidators,
        ...props.validators
    ];

/* eslint-enable complexity */
