import R from 'ramda';
import RS from 'ramdasauce';
import {ERROR, WARNING} from '../utils/ci-utils';
import {isDisabled} from '../utils/components-utils';

export const isToShowMessage = (props, pristine, validationToShow = {message: {show: false}}) => {
    if (props.showParentMessages) {
        return false;
    }

    const isTypeDefined = R.pipe(
        R.prop('type'),
        R.complement(R.isNil)
    );

    const isGenerallyEnabledMessage = !isTypeDefined(props.message) && R.pathOr(false, ['message', 'show'], props);

    const isEnabled = (message) => message.show || isGenerallyEnabledMessage;

    const isShown = (message) => !pristine || isEnabled(message);

    const showMessage = (message) => (message.type === WARNING || message.type === ERROR) && isShown(message);

    const shouldBeShown =
        R.pathSatisfies((message) => isTypeDefined(message) && showMessage(message), ['message']);

    return R.any(shouldBeShown)([props, validationToShow]);
};

// eslint-disable-next-line complexity
export const validateComponent = (props, validators, model, pristine) => {
    if (isDisabled(props.disabled, props.displayMode)) {
        return {valid: true};
    }

    const isValid = ({message}) => message.type !== ERROR;

    if (R.pathSatisfies((text) => !RS.isNilOrEmpty(text), ['message', 'text'], props)) {
        return {
            message: {
                ...props.message,
                show: isToShowMessage(props, pristine)
            },
            valid: isValid(props)
        };
    }

    const evaluatedValidations = R.map((validator) => validator(model), validators);
    const validationToShow = R.find((result) => result.condition, evaluatedValidations);

    const show = isToShowMessage(props, pristine, validationToShow);

    return R.whereEq({condition: true}, validationToShow || {}) ?
        {
            message: {...validationToShow.message, show},
            valid: isValid(validationToShow)
        } :
        {valid: true};
};
