import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {Button, ButtonGroup} from 'react-bootstrap';
import classNames from 'classnames';
import {XlReactWidgetWithPopup} from '../../xl-react-widget/xl-react-widget-with-popup/xl-react-widget-with-popup';
import RU from '../../utils/ramda-utils';
import {getLabel, getValue} from './xl-react-toggle-button-service';
import './xl-react-toggle-button.less';

export class XlReactToggleButton extends PureComponent {
    static propTypes = {
        className: PropTypes.string,
        metadata: PropTypes.arrayOf(PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.object
        ])).isRequired,
        model: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.object
        ]),
        objLabel: PropTypes.string,
        objValue: PropTypes.string,
        onToggle: PropTypes.func.isRequired,
        type: PropTypes.string
    };

    static defaultProps = {type: 'secondary'};

    getButtonClasses(index) {
        const {type, metadata} = this.props;
        const firstButton = R.equals(0, index);
        const lastButton = R.pipe(R.length, R.dec, R.equals(index))(metadata);

        return classNames(
            'xl-react-button',
            `xl-${type}`,
            'xl-react-toggle-button-group',
            {
                'xl-button-left': firstButton,
                'xl-button-middle': !firstButton && !lastButton,
                'xl-button-right': lastButton
            }
        );
    }

    render() {
        const {
            className, metadata, model, onToggle, objValue, objLabel
        } = this.props;

        const buttons = RU.mapIndexed((buttonMetadata, index) => {
            const label = getLabel(buttonMetadata, objLabel);
            const buttonValue = getValue(buttonMetadata, objValue);
            const disabled = R.equals(model, buttonValue);

            const handleOnClick = () => {
                onToggle(buttonValue);
            };
            return <XlReactWidgetWithPopup
                key={index}
                popoverContent={label}
                popoverid={`popover-${index}`}>
                <div>
                    <Button
                        className={this.getButtonClasses(index)}
                        disabled={disabled}
                        key={label}
                        onClick={handleOnClick}
                        style={disabled ? {pointerEvents: 'none'} : {}}
                        title={label}>
                        {label}
                    </Button>
                </div>
            </XlReactWidgetWithPopup>;
        })(metadata);

        const btnGroupWrapper = classNames('xl-react-toggle-button', className);
        return <ButtonGroup className={btnGroupWrapper}>
            {buttons}
        </ButtonGroup>;
    }
}

export default XlReactToggleButton;
