import * as KeyCodes from 'keycode-js';
import R from 'ramda';
import RU from '../../utils/ramda-utils';

export const isKeyUnique = (searchKey, oldKey) => R.pipe(
    R.ifElse(
        R.is(Array),
        R.identity,
        R.keys
    ),
    R.reject(R.equals(oldKey)),
    R.none(R.equals(searchKey))
);

export const defaultState = {
    newKey: '',
    newValue: '',
    showNewKeyValueRow: false
};

export const blurFieldOnSubmit = (field, event) => {
    if (event.keyCode === KeyCodes.KEY_RETURN) {
        field.blur();
    }
};

export const getDerivedState = (props) => {
    if (RU.areDefinedAndDifferent(props.changedValue, props.value)) {
        return {internalValue: props.changedValue};
    } else if (!R.isNil(props.value)) {
        return {internalValue: props.value};
    }
    return null;
};

const createEmptyKeyError = (model) => ({
    emptyKey: true,
    hasNewKeyError: true,
    model: R.clone(model)
});

const createDuplicateKeyError = (trimmedNewKey, model) => ({
    duplicateKey: trimmedNewKey,
    hasNewKeyError: true,
    model: R.clone(model)
});

const createKeyError = (trimmedNewKey, model) => !trimmedNewKey ?
    createEmptyKeyError(model) : createDuplicateKeyError(trimmedNewKey, model);

export const computeNewStateOnNewKeyBlur = (currentState, model) => {
    const {newKey, newValueFocused} = currentState;
    const trimmedNewKey = R.trim(newKey);
    const isNotUnique = !isKeyUnique(trimmedNewKey)(model);

    return R.cond([
        [() => newValueFocused && (!trimmedNewKey || isNotUnique),
            R.always(createKeyError(trimmedNewKey, model))],
        [() => !trimmedNewKey, R.always(defaultState)],
        [R.T, R.always({newKey: trimmedNewKey})]
    ])();
};
