import React from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {ReactSortable} from 'react-sortablejs';
import {XlReactWidgetWithPopup} from '../../xl-react-widget/xl-react-widget-with-popup/xl-react-widget-with-popup';
import {localeMessages} from '../../services/locale-service';
import {nextId} from '../../utils/id-generator';

const SortableList = ({disabled, canSort, indexedItems = [], onChange, remove}) => {
    const handleRemove = (index) => () => remove(index);
    const handleSort = (event) => {
        const {oldIndex, newIndex} = event;
        R.pipe(
            R.move(oldIndex, newIndex),
            R.map(R.path(['model', 0])),
            onChange
        )(indexedItems);
    };

    const removeButton = (index) =>
        disabled ?
            null :
            <i className="xl-icon close-icon" onClick={handleRemove(index)}/>;

    const listItems = R.map((current) => {
        const {message = localeMessages.invalidItem, model: [val, index], valid} = current;
        return <li className="xl-list-row" data-value={val} key={index}>
            {canSort && <span className="xl-icon drag-vertical-icon"/>}
            {!valid && <XlReactWidgetWithPopup
                popoverContent={message}
                popoverid={nextId()}>
                <i className="xl-icon warning-icon"/>
            </XlReactWidgetWithPopup>}
            <span className="text-container" title={val}>{val}</span>
            {removeButton(index)}
        </li>;
    }, indexedItems);

    const disabledList = <ul>
        {listItems}
    </ul>;
    const sortableList = <ReactSortable
        className={canSort && 'ui-sortable'}
        list={indexedItems}
        onEnd={handleSort}
        setList={R.F}
        tag="ul">
        {listItems}
    </ReactSortable>;

    return canSort ? sortableList : disabledList;
};

SortableList.propTypes = {
    canSort: PropTypes.bool,
    disabled: PropTypes.bool,
    indexedItems: PropTypes.array,
    onChange: PropTypes.func,
    remove: PropTypes.func
};

export default SortableList;
