import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {isReadOnly} from '../../utils/display-mode';
import {transformItem} from '../../utils/transform-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {addItem, filterItems} from './xl-react-dip-collection-of-ci-service';
import {XlReactDipCollectionOfCiComponent} from './xl-react-dip-collection-of-ci-component';

export class XlReactDipCollectionOfCi extends Component {
    static propTypes = {
        emptyPlaceholder: PropTypes.string,
        filterVisible: PropTypes.bool,
        handlers: PropTypes.object,
        inputVisible: PropTypes.bool,
        model: PropTypes.array,
        newItemSuffix: PropTypes.string,
        objLabel: PropTypes.string,
        openOnClick: PropTypes.bool,
        sortable: PropTypes.bool,
        sortAutomatically: PropTypes.bool,
        unique: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        context: {searchTerm: ''},
        filterVisible: true,
        inputVisible: true,
        metadata: {},
        objLabel: 'id',
        validators: []
    };

    constructor(props) {
        super(props);
        RU.invoke('onModelChange', [props.model, false], props);
        RU.invoke('register', [this.api], props);
    }

    onAddItem = (item) => {
        const model = addItem(this.props.model, transformItem(item, this.props.objLabel), this.props.unique);
        this.applyNewModel(model, () => this.autoCompleteApi.erase());
    };

    onSortableListChange = (model) => {
        this.applyNewModel([...model].map(transformItem(R.__, this.props.objLabel)));
    };

    onFilterUpdate = (searchTerm) => {
        const {context: {validatedItems}, objLabel} = this.props;
        this.props.mergeContext({searchTerm, visibleItems: filterItems(validatedItems, searchTerm, objLabel)});
    };

    onRemoveItem = (itemIndex) => {
        this.applyNewModel(R.remove(itemIndex, 1, this.props.model));
    };

    api = {focus: () => this.autoCompleteApi.focus()};

    applyNewModel = (model, customCallback = R.F) => {
        this.props.onModelChange(model);
        setTimeout(() => customCallback());
    };

    registerAutoCompleteApi = (autoCompleteApi) => {
        this.autoCompleteApi = autoCompleteApi;
    };

    render() {
        const {
            context: {searchTerm, visibleItems},
            disabled,
            displayMode,
            filterVisible,
            handlers,
            inputVisible,
            message,
            metadata,
            objLabel,
            onMessageChange,
            onValidityChange,
            sortable,
            valid
        } = this.props;

        const readOnly = isReadOnly(displayMode, disabled);

        return <XlReactDipCollectionOfCiComponent {...({
            filterVisible,
            handleAddItem: this.onAddItem,
            handleFilterUpdate: this.onFilterUpdate,
            handleRemoveItem: this.onRemoveItem,
            handlers,
            handleSortableListChange: this.onSortableListChange,
            inputVisible,
            message,
            metadata,
            objLabel,
            onMessageChange,
            onValidityChange,
            readOnly,
            registerAutoCompleteApi: this.registerAutoCompleteApi,
            searchTerm,
            sortable,
            valid,
            visibleItems
        })}
        />;
    }
}
