import React, {Component} from 'react';
import PropTypes from 'prop-types';
import Draggable from 'react-draggable';
import {preventEvent, COLUMN_COUNT} from '../map-table-utils';

import './column-resizer.less';

export const RESIZER_WIDTH = 10;

export default class ColumnResizer extends Component {
    static propTypes = {
        bounds: PropTypes.shape({left: PropTypes.number.isRequired, right: PropTypes.number.isRequired}).isRequired,
        height: PropTypes.number.isRequired,
        leftColumn: PropTypes.string.isRequired,
        onResize: PropTypes.func.isRequired,
        position: PropTypes.number,
        rightColumn: PropTypes.string.isRequired,
        tableWidth: PropTypes.number.isRequired
    };

    static defaultProps = {position: 1};

    static getDerivedStateFromProps = (props, state) => {
        const {tableWidth, position} = props;
        const initialResizerPosition = tableWidth / COLUMN_COUNT * position - RESIZER_WIDTH / 2;
        return {initialResizerPosition, offset: state.offset};
    };

    state = {offset: 0};

    handleOnDrag = (event, {deltaX}) => {
        const {onResize, leftColumn, rightColumn} = this.props;
        onResize({deltaX, leftColumn, rightColumn});
        this.setState({offset: this.state.offset + deltaX});
    };

    render() {
        const {height, bounds} = this.props;
        const {initialResizerPosition, offset} = this.state;
        const position = {x: 0}; // eslint-disable-line id-length
        const style = {
            height,
            left: initialResizerPosition + offset
        };
        return <Draggable
            axis="x"
            bounds={bounds}
            defaultPosition={position}
            onDrag={this.handleOnDrag}
            zIndex={2147483647}>
            <span
                className="column-resize-handle"
                onClick={preventEvent}
                style={style}
            />
        </Draggable>;
    }
}
