import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import {localeMessages} from '../../services/locale-service';
import {InlineKeyInput} from '../input/inline-key-input';

export class InputKey extends PureComponent {
    static propTypes = {
        assignRef: PropTypes.func.isRequired,
        disabled: PropTypes.bool.isRequired,
        handleNewKeyBlur: PropTypes.func.isRequired,
        handleNewKeyChange: PropTypes.func.isRequired,
        handleNewKeyFocus: PropTypes.func.isRequired,
        invalid: PropTypes.bool.isRequired,
        model: PropTypes.string,
        shouldFocus: PropTypes.bool
    };

    render() {
        const {
            assignRef,
            invalid,
            disabled,
            handleNewKeyFocus,
            handleNewKeyChange,
            handleNewKeyBlur,
            model,
            shouldFocus
        } = this.props;

        const inputKeySpan = <span className="input-key">{localeMessages.keyPlaceholder}</span>;

        const customClasses = classNames('key-div', 'new-key-div', {invalid});
        const inputKeyInput = <div className={customClasses}>
            <InlineKeyInput
                assignRef={assignRef}
                name="add-key"
                onBlur={handleNewKeyBlur}
                onChange={handleNewKeyChange}
                placeholder={localeMessages.keyPlaceholder}
                readOnly={disabled}
                shouldFocus={shouldFocus}
                updateCellState={handleNewKeyFocus}
                value={model}
            />
        </div>;

        return disabled ? inputKeySpan : inputKeyInput;
    }
}
