import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {blurFieldOnSubmit} from '../input/input-service';
import {updateState} from './map-table-utils';

export class MapKeyInput extends Component {
    static propTypes = {
        assignRef: PropTypes.func,
        changedValue: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
        className: PropTypes.string,
        onBlur: PropTypes.func,
        onChange: PropTypes.func,
        onKeyDown: PropTypes.func,
        placeholder: PropTypes.string,
        readOnly: PropTypes.bool,
        shouldFocus: PropTypes.bool,
        updateCellState: PropTypes.func,
        value: PropTypes.string.isRequired
    };

    constructor(props) {
        super(props);
        const {changedValue, value} = props;
        if (!R.isNil(changedValue) && value !== changedValue) {
            this.state = {internalValue: props.changedValue};
        } else {
            this.state = {internalValue: props.value};
        }
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        updateState(this, nextProps);
    }

    componentWillUnmount() {
        const {internalValue} = this.state;
        const {value} = this.props;
        if (!R.equals(internalValue, value)) {
            this.props.onBlur(internalValue, value);
        }
    }

    handleBlur = (event) => {
        this.props.onBlur(this.state.internalValue, this.props.value, event.target);
    };

    handleFocus = () => {
        this.props.updateCellState(this.state.internalValue);
    };

    handleValueChange = (event) => {
        RU.invoke('onChange', [event], this.props);
        this.setState({internalValue: event.target.value}, () => {
            this.props.updateCellState(this.state.internalValue);
        });
    };

    handleKeyDown = (event) => {
        blurFieldOnSubmit(this.inputField, event);
    };

    assignRef = (input) => {
        this.inputField = input;
        RU.invoke('assignRef', [input], this.props);
    };

    render() {
        const {readOnly, placeholder, className} = this.props;
        const {internalValue} = this.state;

        return <div className="key-div">
            <input
                className={className}
                onBlur={this.handleBlur}
                onChange={this.handleValueChange}
                onFocus={this.handleFocus}
                onKeyDown={this.handleKeyDown}
                placeholder={placeholder}
                readOnly={readOnly}
                ref={this.assignRef}
                title={internalValue}
                type="text"
                value={internalValue}
            />
        </div>;
    }
}
