import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {SortDirection} from 'react-virtualized';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {filterModel, sortData} from './xl-react-dip-set-of-strings-service';
import SetOfStringsTable from './set-of-strings-table';
import {appendNewKeyValueRow} from './set-of-strings-utils';

import './scrollable-container.less';
import './set-of-strings-table.less';

export class ScrollableContainer extends Component {
    static propTypes = {
        assignRef: PropTypes.func.isRequired,
        assignTableRef: PropTypes.func.isRequired,
        assignTableWrapperRef: PropTypes.func.isRequired,
        cellState: PropTypes.shape({
            column: PropTypes.oneOf(['key']).isRequired,
            row: PropTypes.number.isRequired,
            value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired
        }),
        disabled: PropTypes.bool.isRequired,
        encrypted: PropTypes.bool,
        focus: PropTypes.bool,
        handleCloseButton: PropTypes.func.isRequired,
        handleKeyBlur: PropTypes.func.isRequired,
        handleNewKeyBlur: PropTypes.func.isRequired,
        handleNewKeyChange: PropTypes.func.isRequired,
        handleNewKeyFocus: PropTypes.func.isRequired,
        handleRemoveKey: PropTypes.func.isRequired,
        handleSortChange: PropTypes.func.isRequired,
        hasKeyError: PropTypes.bool,
        message: xlMessageTypeProp,
        model: PropTypes.array.isRequired,
        newKey: PropTypes.string,
        pristine: PropTypes.bool.isRequired,
        searchWord: PropTypes.string,
        showNewKeyValueRow: PropTypes.bool,
        sorting: PropTypes.shape({
            sortBy: PropTypes.oneOf(['key']).isRequired,
            sortDirection: PropTypes.oneOf([SortDirection.DESC, SortDirection.ASC]).isRequired
        }),
        updateCellState: PropTypes.func.isRequired,
        valid: PropTypes.bool.isRequired
    };

    static defaultProps = {valid: true};

    static getDerivedStateFromProps(props, state) {
        const {
            encrypted,
            model,
            searchWord,
            showNewKeyValueRow,
            sorting = {}
        } = props;
        const predicate = R.whereEq({
            model,
            searchWord,
            showNewKeyValueRow,
            sorting
        });
        if (!predicate(state)) {
            const filteredPairs = filterModel(model, searchWord, encrypted);
            const modelForTable = appendNewKeyValueRow(
                props.showNewKeyValueRow,
                sortData(filteredPairs, sorting.sortDirection)
            );

            return {
                model,
                modelForTable,
                searchWord,
                showNewKeyValueRow,
                sorting
            };
        }

        return {};
    }

    state = {};

    handleWrapperBlur = () => {
        this.props.updateCellState();
    };

    assignTableWrapperRef = (ref) => {
        this.props.assignTableWrapperRef(ref);
    };

    assignTableRef = (ref) => {
        this.table = ref;
        this.props.assignTableRef = ref;
    };

    render() {
        const {
            focus,
            handleSortChange,
            sorting: {
                sortBy,
                sortDirection
            } = {}
        } = this.props;

        const {modelForTable} = this.state;

        return <div className="table-wrapper" onBlur={this.handleWrapperBlur} ref={this.assignTableWrapperRef}>
            <SetOfStringsTable
                assignTableRef={this.assignTableRef}
                data={modelForTable}
                focus={focus}
                onSortChange={handleSortChange}
                sortBy={sortBy}
                sortDirection={sortDirection}
                {...this.props}
            />
        </div>;
    }
}
