import R from 'ramda';
import RS from 'ramdasauce';
import {RULE_TYPE} from '../utils/ci-property-rule-types';
import {WARNING} from '../utils/ci-utils';
import {localeMessages} from '../services/locale-service';

export const ruleTypeEq = R.propEq('type');

export const mapPatternRule = (rule) => (model) => ({
    condition: !new RegExp(rule.pattern).test(model),
    message: {
        rule: true,
        show: false,
        text: R.replace(/%s/, model, rule.message),
        type: WARNING
    }
});

export const rangeRuleCondition = (min, max, model) => R.cond([
    [R.complement(isFinite), R.complement(R.test(/^{{.+}}$/))],
    [R.T, R.anyPass([R.lt(R.__, min), R.gt(R.__, max)])]
])(model);

export const mapRangeRule = ({min, max, message}) => (model) => ({
    condition: rangeRuleCondition(min, max, model),
    message: {
        rule: true,
        show: false,
        text: R.replace(/%s/, model, message),
        type: WARNING
    }
});

export const mapPropValidators = R.cond([
    [ruleTypeEq(RULE_TYPE.range), mapRangeRule],
    [R.T, mapPatternRule]
]);

export const createCiPropertyValidators = (props, ciProperty) => {
    const propValidators = R.map((validator) => R.curryN(2, validator)(ciProperty.name), props.validators);
    const inputRulesHintValidators = R.pipe(
        R.pathOr([], ['inputHint', 'rules']),
        R.map(mapPropValidators)
    )(ciProperty);

    const inputRequiredHintValidator = (model) => ({
        condition: RS.isNilOrEmpty(model) && R.pathOr(false, ['inputHint', 'required'], ciProperty),
        message: {
            show: false,
            text: localeMessages.valueIsRequired,
            type: WARNING
        }
    });

    return [...propValidators, ...inputRulesHintValidators, inputRequiredHintValidator];
};
