import R from 'ramda';
import moment from 'moment';
import {hasValue} from '../../utils/components-utils';
import {transformModel} from '../../utils/transform-utils';
import {sortItems} from '../../xl-react-dip/xl-react-dip-collection-of-ci/xl-react-dip-collection-of-ci-service';

export const initialValues = {
    BOOLEAN: () => false,
    CI: () => '',
    DATE: () => new Date(),
    ENUM: () => '',
    INTEGER: () => '',
    LIST_OF_CI: () => [],
    LIST_OF_STRING: () => [],
    MAP_STRING_STRING: () => ({}),
    SET_OF_CI: () => [],
    SET_OF_STRING: () => [],
    STRING: () => ''
};

export const transform = (kind, objLabel, model) => R.cond([
    [R.equals('DATE'), () => moment(model).toDate()],
    [R.equals('LIST_OF_CI'), () => transformModel(objLabel)(model)],
    [R.equals('SET_OF_CI'), () => transformModel(objLabel)(sortItems(model, objLabel))],
    [R.T, () => model]
])(kind);

export const createInitialPropertyModel = ({externalProperties = [], metadataProperty, model, objLabel}) => {
    if (R.contains(metadataProperty.name, externalProperties)) {
        return undefined;
    }
    const {kind} = metadataProperty;
    const fallbackValue = initialValues[kind]();
    const value = hasValue(model) ? model : metadataProperty.default;
    return R.clone(hasValue(value) ? transform(kind, objLabel, value) : fallbackValue);
};

export const createInitialModel = ({externalProperties, ignoredProperties, metadata, model = {}, objLabel = 'id'}) =>
    R.reduce((acc, metadataProperty) =>
        R.contains(metadataProperty.name, ignoredProperties) ? acc : {
            ...acc,
            [metadataProperty.name]: createInitialPropertyModel({
                externalProperties,
                metadataProperty,
                model: model[metadataProperty.name],
                objLabel
            })
        }, model, metadata.properties);
