import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import {Markup} from 'interweave';
import {xlMessageTypeProp} from '../xl-props/xl-message-type-prop';
import {ERROR, WARNING} from '../utils/ci-utils';
import {XlReactWidgetLabel} from '../xl-react-widget/xl-react-widget-label/xl-react-widget-label';

export const formItemControlExposedProps = {
    customControlClasses: PropTypes.object,
    description: PropTypes.string,
    disabled: PropTypes.bool,
    inputIconRenderer: PropTypes.func,
    label: PropTypes.string,
    message: xlMessageTypeProp,
    messageHandler: PropTypes.func,
    required: PropTypes.bool,
    requiredDirection: PropTypes.oneOf(['left', 'right'])
};

export class XlReactFormItemControl extends PureComponent {
    static propTypes = {
        children: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
        controlClass: PropTypes.string,
        ...formItemControlExposedProps
    };

    descriptionOrErrorMsg = () => {
        const getVisibleText = (props, hasVisibleMessage) => {
            const {description, message} = props;

            return hasVisibleMessage ? R.prop('text', message) : description;
        };

        const {description, messageHandler, message} = this.props;
        const hasVisibleMessage = () => R.prop('text', message) && R.prop('show', message) && !messageHandler;
        const showText = () => hasVisibleMessage() || description;
        const visibleText = getVisibleText(this.props, hasVisibleMessage());

        const isError = () => hasVisibleMessage() && message.type === ERROR;
        const isWarning = () => hasVisibleMessage() && message.type === WARNING;

        const labelClasses = classNames(
            {'xl-react-components-description': true},
            {warning: isWarning()},
            {error: isError()}
        );
        const getContent = () => R.is(String, visibleText) ? <Markup content={visibleText}/> : visibleText;
        return showText() ? <div className={labelClasses}>{getContent()}</div> : null;
    };

    render() {
        const {
            disabled, inputIconRenderer, label, required, requiredDirection, controlClass
        } = this.props;

        const labelElement = label ? <XlReactWidgetLabel metadata={{label, required, requiredDirection}}/> : null;

        const componentClass = classNames(`xl-react-components ${controlClass}`, {'is-disabled': disabled});

        const componentBodyClass = classNames({
            'xl-react-components-input': label,
            'xl-react-components-input-full': !label,
            ...this.props.customControlClasses || {}
        });

        const renderIcon = () => inputIconRenderer ? inputIconRenderer() : null;

        return <div className={componentClass}>
            {labelElement}
            <div className={componentBodyClass}>
                {renderIcon()}
                {this.props.children}
                {this.descriptionOrErrorMsg()}
            </div>
        </div>;
    }
}
