import PropTypes from 'prop-types';
import React, {PureComponent} from 'react';

import classNames from 'classnames';

import './xl-react-widget-alert-message.less';

export class XlReactWidgetAlertMessage extends PureComponent {
    static propTypes = {
        children: PropTypes.any,
        hide: PropTypes.bool,
        internalState: PropTypes.string.isRequired,
        metadata: PropTypes.object,
        onDismiss: PropTypes.func,
        required: PropTypes.bool,
        showClose: PropTypes.bool
    };

    static defaultProps = {
        internalState: 'error',
        metadata: {}
    };

    handleDismiss = () => {
        this.props.onDismiss();
    };

    render() {
        const {
            internalState,
            metadata,
            showClose,
            children,
            hide
        } = this.props;

        const componentClass = classNames({
            'xl-react-alert-error': internalState === 'error',
            'xl-react-alert-message': true,
            'xl-react-alert-success': internalState === 'success',
            'xl-react-alert-warning': internalState === 'warning',
            'xl-react-components': true
        });

        const createCloseBtn = () =>
            showClose && <i className="xl-icon close-icon" onClick={this.handleDismiss}/>;

        return (
            hide ? null : <div className={componentClass}>
                {metadata.indentForLabel && <div className="indent-for-label"/>}
                {children}
                {createCloseBtn()}
            </div>
        );
    }
}
