import R from 'ramda';
import React from 'react';
import {transformItem, transformItemOr} from '../../../utils/transform-utils';
import RU from '../../../utils/ramda-utils';
import {reasonTypeOfAddingCandidate} from '../reason-type-of-adding-candidate';
import {focusWithTimeout, shouldGetSuggestions} from '../../../utils/behaviour-utils';

const renderOption = (component, objLabel, value) => {
    const optionItem = transformItem(component, objLabel);
    const valueItem = transformItemOr('', value, objLabel);
    const index = R.toLower(optionItem).indexOf(R.toLower(valueItem));
    if (R.gt(index, -1)) {
        return <span>
            <div>
                {optionItem.slice(0, index)}
                <strong>{optionItem.slice(index, index + valueItem.length)}</strong>
                {optionItem.slice(index + valueItem.length)}
            </div>
        </span>;
    }

    return optionItem;
};

export const defaultSuggestionBehavior = {
    handleOnFocus: R.curry((component, event) => {
        const {handleGetSuggestions, inputProps, model} = component.props;
        const {typing} = component.state;
        if (shouldGetSuggestions(component)) {
            handleGetSuggestions(typing ? model : '', reasonTypeOfAddingCandidate.onInputFocus);
            component.setState({isCollapsed: false, isFocused: true});
        }
        RU.invoke('onFocus', [event], inputProps);
    }),
    handleOnSuggestionClick: R.curry((component, event, suggestion) => {
        const {handleSuggestionSelected, model, objLabel} = component.props;
        const newValue = !R.is(Object, model) ? transformItem(suggestion, objLabel) : suggestion;
        handleSuggestionSelected(event, {suggestion});
        component.notifyOnChange(event, newValue, reasonTypeOfAddingCandidate.onSuggestionClick);
        component.setState({inputValue: transformItemOr('', suggestion, objLabel)});
        component.closeSuggestions();
        focusWithTimeout(component, event, defaultSuggestionBehavior);
    }),
    renderSuggestion: R.curry((component, data, value) => {
        const {objLabel} = component.props;
        return <div className="yt-option" title={transformItem(data, objLabel)}>
            {renderOption(data, objLabel, value)}
        </div>;
    })
};
