import R from 'ramda';
import React from 'react';
import PropTypes from 'prop-types';
import {Panel} from 'react-bootstrap';
import {localeMessages} from '../../../services/locale-service';
import {patchType} from '../patch-type';
import {XlReactWidgetMonacoEditor}
    from '../../xl-react-widget-monaco/xl-react-widget-monaco-editor/xl-react-widget-monaco-editor';
import './editor-panel.less';

const EditorPanel = (props) => {
    const {
        model,
        onModelChange,
        type
    } = props;
    const options = {
        lineNumbersMinChars: 3,
        readOnly: R.propOr(false, 'readOnly', props),
        scrollBeyondLastLine: false,
        selectOnLineNumbers: true
    };

    return (
        <Panel className="editor-panel">
            <Panel.Heading className="panel-heading">
                <div className="header-label">
                    {localeMessages.patchDictionaryPatchWithPanelTitle}
                </div>
            </Panel.Heading>
            <XlReactWidgetMonacoEditor
                language={type}
                onChange={onModelChange}
                options={options}
                theme="vs"
                value={model}
            />
        </Panel>
    );
};

EditorPanel.propTypes = {
    focusOnMount: PropTypes.bool,
    model: PropTypes.string.isRequired,
    onModelChange: PropTypes.func.isRequired,
    readOnly: PropTypes.bool,
    type: patchType
};

export const XlEditorPanel = React.memo(EditorPanel);
