import {call, delay, put, select} from 'redux-saga/effects';
import {INPUT_DEBOUNCE_TIME} from '../xl-react-widget-redux-table-constants';
import tableActions from '../xl-react-widget-redux-table-actions';
import {extractTotal} from '../../../utils/http-utils';
import {getSortingWithSortKey, getTableById} from '../xl-react-widget-table-selectors';
import {buildFilterQuery} from '../query-builder-utils';
import {paginateSaga} from './xl-react-widget-redux-table-pagination-saga';

export function* withDelay(effect, timeout = INPUT_DEBOUNCE_TIME) {
    yield delay(timeout);
    yield effect;
}

export function* fetchData(tableId, api, metaData) {
    const {filters, paging} = yield select(getTableById, tableId);
    const filterQuery = buildFilterQuery(filters);
    const sortQuery = yield select(getSortingWithSortKey, tableId);
    try {
        const {data, headers} = yield call(api, filterQuery, paging, sortQuery, metaData);
        yield put(tableActions.show({
            data,
            tableId,
            total: extractTotal(headers)
        }));
    } catch (error) {
        yield put(tableActions.show({
            error,
            tableId
        }));
    }
}

export function* withSpinner(tableId, effect) {
    try {
        yield put(tableActions.changeLoadingState(tableId, true));
        yield effect;
        yield put(tableActions.changeLoadingState(tableId, false));
    } catch (error) {
        yield put(tableActions.changeLoadingState(tableId, false));
    }
}

export function* initializeSaga(action) {
    if (action.api) {
        yield call(withSpinner, action.tableId, call(fetchData, action.tableId, action.api));
    }
}

export function* refreshSaga({api, tableId}) {
    const {paging} = yield select(getTableById, tableId);
    if (paging) {
        const {resultsPerPage} = paging;
        yield call(paginateSaga, {
            api,
            page: 1,
            resultsPerPage,
            tableId
        });
    } else {
        yield call(withSpinner, tableId, call(fetchData, tableId, api));
    }
}
