import classNames from 'classnames';
import React from 'react';
import PropTypes from 'prop-types';
import './xl-react-widget-string-monaco-editor.less';
import {validationMessageClasses} from '../../utils/css-utils';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {XlReactWidgetMonacoEditor}
    from '../xl-react-widget-monaco/xl-react-widget-monaco-editor/xl-react-widget-monaco-editor';

const ReactWidgetStringMonacoEditor = (props) => {
    const editorDidMount = (editor) => {
        editor.onDidFocusEditorText(props.onFocus);
        editor.onDidBlurEditorText(props.onBlur);
    };

    const componentOptions = {
        ...props.options,
        ...props.disabled ? {readOnly: true} : {readOnly: false}
    };

    const handleChange = (value) => {
        props.onModelChange({target: {value}});
    };

    const componentClasses = classNames(
        props.className,
        {focused: !props.disabled && props.focused},
        {'readonly-input': props.disabled},
        validationMessageClasses(props.valid, props.message)
    );

    return <div className={componentClasses}>
        <XlReactWidgetMonacoEditor
            editorDidMount={editorDidMount}
            language={props.language}
            onChange={handleChange}
            options={componentOptions}
            ref={props.assignRef}
            theme={'vs'}
            value={props.model}
        />
    </div>;
};

ReactWidgetStringMonacoEditor.propTypes = {
    assignRef: PropTypes.func.isRequired,
    className: PropTypes.string,
    disabled: PropTypes.bool,
    focused: PropTypes.bool,
    language: PropTypes.string,
    message: xlMessageTypeProp,
    model: PropTypes.string,
    name: PropTypes.string,
    onBlur: PropTypes.func,
    onFocus: PropTypes.func,
    onModelChange: PropTypes.func,
    options: PropTypes.object,
    placeholder: PropTypes.string,
    required: PropTypes.bool,
    size: PropTypes.string,
    valid: PropTypes.bool
};

ReactWidgetStringMonacoEditor.defaultProps = {language: 'json'};

export const XlReactWidgetStringMonacoEditor = React.memo(ReactWidgetStringMonacoEditor);
