import PropTypes from 'prop-types';
import R from 'ramda';
import React, {Component} from 'react';
import RU from '../../utils/ramda-utils';
import {localeMessages} from '../../services/locale-service';
import {getModelValue, isDisabled, resetPristinity} from '../../utils/components-utils';
import {isSuggestionClick} from '../xl-react-widget-autocomplete/reason-type-of-adding-candidate';
import {XlReactWidgetContainerProps} from '../xl-react-widget-props';
import {
    computeInitialState,
    computeNextPropsState,
    computeNextState,
    getValidators
} from '../../services/component-service';
import {transformItem} from '../../utils/transform-utils';
import {ERROR} from '../../utils/ci-utils';
import {tagsAutocompleteSkins} from './xl-react-widget-tags-skins';
import {
    appendNewSuggestion,
    getIndex,
    getItemsToSelect,
    getSuggestionBehavior,
    inputTypes,
    removeExistingSuggestion,
    selectRemainingSuggestions,
    toggleChecked,
    transformModelForInputTags
} from './xl-react-widget-tags-service';
import {XlReactWidgetTagsComponent} from './xl-react-widget-tags-component';
import {XlReactWidgetTagsModelProps} from './xl-react-widget-tags-model-props';

export class XlReactWidgetTags extends Component {
    static propTypes = {
        allowAddOnlyUniqueSuggestion: PropTypes.bool,
        handlers: PropTypes.object,
        inputType: PropTypes.oneOf(R.values(inputTypes)),
        metadata: PropTypes.object,
        model: XlReactWidgetTagsModelProps,
        objLabel: PropTypes.string,
        onRemove: PropTypes.func,
        showNumberOfRemainingTags: PropTypes.bool,
        skin: PropTypes.oneOf(R.values(tagsAutocompleteSkins)),
        tag: PropTypes.string,
        tagTransform: PropTypes.func,
        ...XlReactWidgetContainerProps
    };

    static defaultProps = {
        inputType: inputTypes.TEXT,
        metadata: {},
        showNumberOfRemainingTags: true,
        skin: tagsAutocompleteSkins.CHECKBOX,
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {suggestions: [], tag: ''};
    }

    UNSAFE_componentWillMount() {
        const model = getModelValue(this.props, 'model', []);
        computeInitialState(this, this.props, this.getTagsValidators(this.props), model);
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, this.getTagsValidators(nextProps));
    }

    handleModelChange = (valuesList) => {
        const model = transformModelForInputTags(valuesList);
        this.applyNewValue(model);
        if (this.isAutocomplete()) {
            this.childApi.revealSuggestions();
        }
    };

    handleTextInputChange = (event) => {
        this.setState({tag: event.target.value});
    };

    handleAutocompleteSelectAll = (suggestions) => {
        this.applyNewValue([
            ...this.props.model,
            ...selectRemainingSuggestions(
                this.props.objLabel,
                this.props.model,
                suggestions,
                this.props.tagTransform
            )
        ], {customCallbacks: [this.childApi.blur]});
    };

    handleAutocompleteClearAll = () => {
        this.applyNewValue([], {customCallbacks: [this.childApi.blur]});
    };

    handleSuggestionSelection = (item, model, objLabel, tagTransform) =>
        R.find(R.propEq('value', transformItem(item, objLabel)))(model) ?
            removeExistingSuggestion(item, model, objLabel) :
            appendNewSuggestion(item, model, objLabel, tagTransform);

    handleBatchSuggestionSelection = (items, stateModel, objLabel, tagTransform) => {
        let model = stateModel;
        R.map((item) => {
            model = this.handleSuggestionSelection(item, model, objLabel, tagTransform);
            return item;
        })(items);
        return model;
    };

    onSuggestionSelection = (item, suggestions, callbacks, event) => {
        if (R.propOr(false, 'shiftKey')(event) && this.getBatchStartNode()) {
            const startIndex = getIndex(this.getBatchStartNode(), suggestions, this.props.objLabel);
            const stopIndex = getIndex(item, suggestions, this.props.objLabel);
            const suggestionsToSelect = getItemsToSelect(startIndex, stopIndex, suggestions);
            this.notify(
                this.handleBatchSuggestionSelection(
                    suggestionsToSelect,
                    this.state.model,
                    this.props.objLabel,
                    this.props.tagTransform
                ), callbacks
            );
        } else {
            this.notify(
                this.handleSuggestionSelection(
                    item,
                    this.state.model,
                    this.props.objLabel,
                    this.props.tagTransform
                ), callbacks
            );
        }
        this.setBatchStartNode(item);
        this.api.focus();
    };

    addCandidates = (addCandidatesHandler) => ({metadata, reason, value}) => {
        const {objLabel} = this.props;
        const {model} = this.state;
        if (isSuggestionClick(reason)) {
            return Promise.resolve(this.state.suggestions)
                .then(R.map(toggleChecked(model, objLabel)));
        }
        return addCandidatesHandler({metadata, objLabel, reason, value})
            .then((suggestions) => {
                this.setState({suggestions});
                return suggestions;
            })
            .then(R.map(toggleChecked(model, objLabel)));
    };

    getHandlers = (props) => R.pipe(
        R.path(['handlers', props.metadata.name]),
        R.evolve({addCandidates: this.addCandidates})
    )(props);

    setBatchStartNode = (item) => {
        this.clickedSuggestion = item;
    };

    getBatchStartNode = () => this.clickedSuggestion;

    isAutocomplete = () => this.props.inputType === inputTypes.AUTOCOMPLETE;

    applyNewValue = (model, config) => {
        computeNextState(this, this.props, this.getTagsValidators(this.props), model, config);
        this.setState({tag: ''});
    };

    assignRef = (input) => {
        this.inputField = input;
    };

    getSubActions = () => [
        {
            action: this.handleAutocompleteSelectAll,
            isDisabled: () => this.childApi &&
                R.equals(
                    R.length(this.childApi.getSuggestions()),
                    R.length(this.state.model)
                ),
            label: localeMessages.selectAllLabel
        },
        {
            action: this.handleAutocompleteClearAll,
            isDisabled: () => R.isEmpty(this.props.model),
            label: localeMessages.clearAllLabel
        }
    ];

    registerChildApi = (childApi) => {
        this.childApi = childApi;
    };

    notify(model, customCallbacks = []) {
        computeNextState(this, this.props, getValidators(this.props), model, {customCallbacks});
    }

    getTagsValidators = (props) => getValidators(props, [(model) => {
        const duplicate = !R.isNil(model) && RU.findDuplicateElement(model);
        return {
            condition: Boolean(duplicate),
            message: {show: false, text: duplicate && localeMessages.duplicateTag(duplicate), type: ERROR}
        };
    }]);

    api = {
        blur: () => this.inputField.blur(),
        focus: () => this.inputField.focus(),
        getMessage: () => R.pathOr(['message', 'text'], this.state),
        isPristine: () => this.state.pristine,
        resetInput: (input = '') => this.setState({tag: input}),
        resetPristinity: (config) => resetPristinity(this, this.props, config)
    };

    addTagOnEnter = (event, api) => {
        event.preventDefault();
        if (this.props.skin === tagsAutocompleteSkins.CHECKBOX) {
            api.erase();
            const {tag} = this.state;
            this.applyNewValue([
                ...this.props.model,
                {
                    label: tag,
                    value: tag
                }
            ], {customCallbacks: [this.childApi.blur]});
        }
    };

    render() {
        const {
            allowAddOnlyUniqueSuggestion, className, displayMode, inputType, metadata,
            onValidityChange, objLabel, showNumberOfRemainingTags, skin, tagTransform
        } = this.props;

        const {description, label, required, requiredDirection} = this.props.metadata;
        const {message, model, tag, valid} = this.state;
        const autocompleteMetadata = R.omit(['default', 'description', 'label', 'required'], metadata);
        const disabled = isDisabled(this.props.disabled, displayMode);
        const suggestionBehavior = getSuggestionBehavior(skin);

        return <XlReactWidgetTagsComponent {...({
            addTagOnEnter: this.addTagOnEnter,
            allowAddOnlyUniqueSuggestion,
            assignRef: this.assignRef,
            autocompleteMetadata,
            className,
            description,
            disabled,
            handleAutocompleteSelectAll: this.handleAutocompleteSelectAll,
            handleModelChange: this.handleModelChange,
            handlers: this.getHandlers(this.props),
            handleSuggestionSelection: this.onSuggestionSelection,
            handleTextInputChange: this.handleTextInputChange,
            inputType,
            label,
            message,
            model,
            objLabel,
            onValidityChange,
            registerChildApi: this.registerChildApi,
            required,
            requiredDirection,
            showNumberOfRemainingTags,
            subActions: this.getSubActions(),
            suggestionBehavior,
            tag,
            tagTransform,
            valid
        })}
        />;
    }
}
