import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import {OverlayTrigger, Popover} from 'react-bootstrap';
import R from 'ramda';
import uuid from 'uuid-v4';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {validationMessageClasses} from '../../utils/css-utils';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';
import {displayAmPmLabel, displayHours, displayMinutes} from './xl-react-widget-time-service';

import './xl-react-widget-time.less';

export class XlReactWidgetTimeComponent extends PureComponent {
    static propTypes = {
        assignRef: PropTypes.func,
        description: PropTypes.string,
        disabled: PropTypes.bool,
        handleAddHour: PropTypes.func,
        handleAddMinute: PropTypes.func,
        handleChangeAmPm: PropTypes.func,
        handleDeductHour: PropTypes.func,
        handleDeductMinute: PropTypes.func,
        label: PropTypes.string,
        message: xlMessageTypeProp,
        model: PropTypes.object,
        onValidityChange: PropTypes.func,
        valid: PropTypes.bool
    };

    static defaultProps = {
        handleAddHour: R.F,
        handleAddMinute: R.F,
        handleChangeAmPm: R.F,
        handleDeductHour: R.F,
        handleDeductMinute: R.F
    };

    constructor(props) {
        super(props);
        this.popoverId = uuid();
    }

    amPmLabel = () => displayAmPmLabel(moment(this.props.model).hours());

    changeElementButton = (direction, className, clickHandler) => <div className="col-xs-4">
        <button
            className={`xl-icon chevron-${direction}-icon ${className}`}
            onClick={clickHandler}
            type="button"
        />
    </div>;

    popover = () => <Popover id={this.popoverId}>
        <div className="xl-react-widget-time-content">
            <div className="row">
                {this.changeElementButton('up', 'plus-hour', this.props.handleAddHour)}
                {this.changeElementButton('up', 'plus-minute', this.props.handleAddMinute)}
                <div className="col-xs-4"/>
            </div>
            <div className="row">
                <div className="col-xs-4 hour-time">
                    {this.showHours()}
                </div>
                <div className="col-xs-4 minute-time">
                    {this.showMinutes()}
                </div>
                <div className="col-xs-4 am-pm">
                    <button
                        onClick={this.props.handleChangeAmPm}
                        type="button">
                        {this.amPmLabel()}
                    </button>
                </div>
            </div>
            <div className="row">
                {this.changeElementButton('down', 'minus-hour', this.props.handleDeductHour)}
                {this.changeElementButton('down', 'minus-minute', this.props.handleDeductMinute)}
                <div className="col-xs-4"/>
            </div>
        </div>
    </Popover>;

    showHours = () => displayHours(moment(this.props.model).hours());

    showMinutes = () => displayMinutes(moment(this.props.model).minutes());

    showTime = () => `${this.showHours()}:${this.showMinutes()} ${this.amPmLabel()}`;

    render() {
        const {
            assignRef, description, disabled, message, label, valid
        } = this.props;

        const handleOnChange = R.F;

        const timeInputPart = <div className="time-input">
            <i className="xl-icon pending-icon"/>
            <input
                disabled={disabled}
                onChange={handleOnChange}
                ref={assignRef}
                type="text"
                value={this.showTime()}
            />
        </div>;

        const timeInput = () => {
            if (disabled) {
                return timeInputPart;
            }
            return <OverlayTrigger
                overlay={this.popover()}
                placement="bottom"
                rootClose={true}
                trigger="click">
                {timeInputPart}
            </OverlayTrigger>;
        };

        const customControlClasses = validationMessageClasses(valid, message);

        return <XlReactFormItemControl {...({
            controlClass: 'xl-react-widget-time',
            customControlClasses,
            description,
            label,
            message
        })}>
            {timeInput()}
        </XlReactFormItemControl>;
    }
}
