import React, {Component} from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import {getModelValue, isComponentDisabled, resetPristinity} from '../../utils/components-utils';
import {XlReactWidgetContainerProps} from '../xl-react-widget-props';
import {
    computeInitialState,
    computeNextState,
    computeNextPropsState,
    getValidators
} from '../../services/component-service';
import {XlReactWidgetTimeComponent} from './xl-react-widget-time-component';
import {addHour, addMinute, changeAmPm, deductHour, deductMinute} from './xl-react-widget-time-service';

export class XlReactWidgetTime extends Component {
    static propTypes = {
        metadata: PropTypes.object,
        model: PropTypes.object,
        ...XlReactWidgetContainerProps
    };

    static defaultProps = {metadata: {}, validators: []};

    constructor(props) {
        super(props);
        this.state = {};
    }

    UNSAFE_componentWillMount() {
        const model = getModelValue(this.props, 'model', new Date());
        const datetime = model.toDate ? model.toDate() : model;
        computeInitialState(this, this.props, getValidators(this.props), datetime);
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps));
    }

    api = {
        focus: () => this.inputField.focus(),
        resetPristinity: (config) => resetPristinity(this, this.props, config)
    };

    applyChanges(model) {
        computeNextState(this, this.props, getValidators(this.props), model.toDate());
    }

    assignRef = (input) => {
        this.inputField = input;
    };

    render() {
        const {metadata: {description, label}, onValidityChange} = this.props;
        const {model, message, valid} = this.state;

        const changeTime = (what, how) => () => {
            const modifiedModel = moment(model);
            modifiedModel[what](how(modifiedModel[what]()));
            this.applyChanges(modifiedModel);
        };

        return <XlReactWidgetTimeComponent
            assignRef={this.assignRef}
            description={description}
            disabled={isComponentDisabled(this.props)}
            handleAddHour={changeTime('hours', addHour)}
            handleAddMinute={changeTime('minutes', addMinute)}
            handleChangeAmPm={changeTime('hours', changeAmPm)}
            handleDeductHour={changeTime('hours', deductHour)}
            handleDeductMinute={changeTime('minutes', deductMinute)}
            label={label}
            message={message}
            model={model}
            onValidityChange={onValidityChange}
            valid={valid}
        />;
    }
}
