# XlReactWidgetTree

A component for displaying tree like structures or lists. As the list is a tree which contain
only root nodes. This component is based on infinite tree which don't render all nodes, but only visible ones.
It makes it possible to work with millions of nodes with a great performance. The look and feels of nodes
is customizable through rowRenderer. 

## Summary

* [Exports](#exports)
* [Playground](#playground)
* [Properties](#properties)
* [Example](#example)

### Exports
```text
- XlReactWidgetTree
```

### Playground
* [Tree Widget](https://xebialabs.github.io/xl-react-components/kitchen-sink.html#/widget/widgetTree1)
* [Lazy Tree Widget](https://xebialabs.github.io/xl-react-components/kitchen-sink.html#/widget/widgetTree2)

### Properties

- **autoOpen** - (bool) - if true, opens all nodes.
- **blocksInCluster** - (number) - Default value is 4. Specifies the amount of blocks in cluster. When scroll reaches last block - content replaces with next cluster.
- **className** - (string) - Specifies the class name of the tree component.
- **dndHandlers** - (object) - provides options to work with drag and drop functionality. Fires when
    - **onDrag** - (func) - a dragged item (element or text selection) is dragged.
    - **onDragEnd** - (func) - a drag operation ends (such as releasing a mouse button or hitting the Esc key)
    - **onDragEnter** - (func) - a dragged item enters a valid drop target. 
    - **onDragExit** - (func) - an element is no longer the drag operation's immediate selection target.
    - **onDragLeave** - (func) - a dragged item leaves a valid drop target.
    - **onDragOver** - (func) - a dragged item is being dragged over a valid drop target, every few hundred milliseconds.
    - **onDragStart** - (func) - the user starts dragging an item.
    - **onDrop** - (func) - an item is dropped on a valid drop target.
- **dontSelectClasses** - (array of strings) - CSS classes which makes nodes not selectable. That can be
useful, when it is possible to select only a certain group of nodes at a time. Or if certain nodes never 
can be selected, or if user doesn't have permissions on certain nodes.  
- **droppableOptions** - (boolean or object) - By default is false (not droppable). Makes tree nodes droppable.
    - **accept** - (function) - By default `(event, options) => true`. Controls which draggable elements are accepted by the droppable.
    - **drop** - (function) - By default `function(event, options) {}`.  Triggered when an accepted draggable is dropped on the droppable.
    - **hoverClass** - (string) - By default `""`. If specified, the class will be added to the droppable while an acceptable draggable is being hovered over the droppable.
- **loadNodes** - (function) - By default `null`. Loads nodes on demand.
- **model** - (array) - Array of nodes treated as an initial data for the tree. 
- **noDataText** - (string) - By default `No data`. Displayed text in case when tree data is empty.
- **nodeHandlers** - (object) - Shape of the object is:  `{ onCloseNode: PropTypes.func, onOpenNode: PropTypes.func}`.
You can provide 1 or 2 handlers to listen events when node is expanded or collapsed. As a parameter you'll receive the full
node.
- **onClick** - (function) - That's a handler which is triggered whenever any part of tree is clicked. 
Therefore you have only 1 handler which reacts on click, instead of N listeners attached to each tree node.
- **onDoubleClick** - (function) - Same as `onClick` but do double click.
- **onNodesSelected``** - (function) - That's handler is triggered in 3 situations: 
    * when single node selected
    * when multiple nodes selected (multi clicked nodes)
    * when nodes selected in batch (Ctrl/Cmd + Shift)   
- **rowRenderer** - (function) - This is a required function which will be triggered to render the node.
There is no default renderer. But there are some predefined renderer which you can use:
    * xlReactTreeRowRenderer
    * xlReactIconRenderer
    * XlReactTreeRowComponent
    * xlReactTreeRowIconRenderer
- **selectable** - (boolean) - By default `true`. If set to `false` it won't be possible to select any node.
Can be useful in case of view only mode.
- **selectedNodes** - (array of strings) - This is an array of node ids which will be highlighted to represent
selected nodes.
- **selectionMode** - (enum 'single' or 'multi') - Default value `single`. If set to 'single', 
selection of second node will deselect first node.
- **treeRef** - (function) - Callback which will be triggered and assigns the reference to a tree element. You
will need it in case of direct access of infinite-tree API methods.

### Example

```javascript
const assignTree = (component) => {
  if (component) {
      this.tree = component.tree;
  }
};

const reportPlanRowRenderer = (node, treeOptions) => {
    const titleSelector = (rowNode) => rowNode.name;
    return <XlReactTreeRowComponent
            node={node}
            titleSelector={titleSelector}
            togglerClass={treeOptions.togglerClass}>
            {treeRowContent}
        </XlReactTreeRowComponent>;
};

const loadSteps = (parentNode, done) => {
    done();
    this.props.loadSteps(parentNode.id);
};

<XlReactWidgetTree
    className="report-plan-tree"
    loadNodes={loadSteps}
    onNodesSelected={handleNodesSelected}
    rowRenderer={reportPlanRowRenderer}
    selectedNodes={this.props.selectedNodes}
    treeRef={assignTree}
/>;
```
