import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import classNames from 'classnames';
import {isNodeOpened, getNodeTogglerClass} from '../xl-react-widget-tree-service';

export class XlReactTreeRowComponent extends Component {
    static propTypes = {
        children: PropTypes.oneOfType([PropTypes.object, PropTypes.array]),
        draggable: PropTypes.bool,
        node: PropTypes.object.isRequired,
        togglerClass: PropTypes.string.isRequired
    };

    static findDataProps = R.pickBy((val, key) => R.startsWith('data-', key));

    render() {
        const {draggable, node, togglerClass} = this.props;
        const {
            id, children, state, title, parent, customClasses
        } = node;

        const dataProps = XlReactTreeRowComponent.findDataProps(this.props);

        const {depth, path, total, selected = false} = state;
        const childrenLength = R.length(children);

        const nodeTogglerClasses = classNames(getNodeTogglerClass(togglerClass, node));

        return <div
            className={classNames(
                'infinite-tree-item',
                {'infinite-tree-selected': selected},
                customClasses
            )}
            data-children={childrenLength}
            data-depth={depth}
            data-expanded={isNodeOpened(node)}
            data-id={id}
            data-parent-type={parent.type}
            data-path={path}
            data-selected={selected}
            data-total={total}
            draggable={draggable}
            title={title}
            {...dataProps}>
            <div className="infinite-tree-node" style={{marginLeft: depth * 18}}>
                <a className={nodeTogglerClasses}/>
                {this.props.children}
            </div>
        </div>;
    }
}
