import R from 'ramda';
import webpack from 'webpack';
import HtmlWebpackPlugin from 'html-webpack-plugin';
import paths from '../../utils/paths';
import {dependencies} from '../../../package.json';
import serverPort from '../../utils/port';
import hostname from '../../utils/hostname';
import common from './webpack.config.common';

export default {
    devServer: {
        contentBase: '/devDist/',
        hot: true,
        port: serverPort,
        publicPath: '/'
    },
    devtool: 'inline-source-map',
    entry: {
        polyfills: './app/polyfills.js',
        main: [
            'eventsource-polyfill',
            'react-hot-loader/patch',
            `webpack-dev-server/client?http://${hostname}:${serverPort}`,
            'webpack/hot/only-dev-server',
            'react-dom/test-utils',
            './app/index',
            'bean'
        ],
        vendor: R.keys(dependencies)
    },
    mode: 'development',
    module: {
        rules: [
            {
                test: require.resolve('jquery'),
                use: ['expose-loader?jQuery', 'expose-loader?$']
            },
            {
                test: require.resolve('react-dom/test-utils'),
                use: ['expose-loader?ReactTestUtils']
            },
            {
                test: /\.ya?ml$/,
                use: 'raw-loader'
            },
            {
                exclude: /(node_modules|tmp)/,
                loader: 'babel-loader',
                options: {
                    // This is a feature of `babel-loader` for webpack (not Babel itself).
                    // It enables caching results in ./node_modules/.cache/babel-loader/
                    // directory for faster rebuilds.
                    cacheDirectory: true,
                    plugins: ['react-hot-loader/babel']
                },
                test: /\.js$/
            },
            ...common.rulesConfig
        ]
    },
    node: {module: 'empty'},
    optimization: {
        splitChunks: {
            cacheGroups: {
                commons: {
                    chunks: 'all',
                    name: 'vendor',
                    test: /[\\/]node_modules[\\/]/
                },
                default: false
            }
        }
    },
    output: {
        filename: '[name]-[hash].js',
        path: paths.devDistDir,
        publicPath: '/'
    },
    plugins: [
        new webpack.HotModuleReplacementPlugin(),
        new webpack.NoEmitOnErrorsPlugin(),
        new HtmlWebpackPlugin({
            template: './app/index.ejs',
            title: 'UI Components'
        })
    ]
};
