import {Action, Expectation} from 'protractor-base-dsl';

const findSampleLine = (lineNumber) => `.xl-react-widget-patch-sample .infinite-tree-item[data-id="${lineNumber}"]`;
const findSampleLineText = (lineNumber) => `${findSampleLine(lineNumber)} pre`;

const expectLineContent = (lineNumber, expectedText) => {
    Expectation.textEquals(findSampleLineText(lineNumber), expectedText);
};

const expectedNoSelectedLine = () => {
    const sampleLineSelector = '.xl-react-widget-patch-sample .infinite-tree-item.infinite-tree-selected';
    Expectation.notDisplayed(sampleLineSelector);

    const displayedSelectedLinesSelector = '.testBindings .selected-lines.value';
    Expectation.textEquals(displayedSelectedLinesSelector, '[]');
};

const expectedSelectedLine = (...lines) => {
    const selectors = lines.map(findSampleLine);
    selectors.forEach((selector) => Expectation.displayed(selector));
    const displayedSelectedLinesSelector = '.testBindings .selected-lines.value';
    Expectation.textEquals(displayedSelectedLinesSelector, JSON.stringify(lines.map((line) => line.toString())));
};

const selectLine = (lineNumber) => Action.jsClick(findSampleLine(lineNumber));

const hoverLine = (lineNumber) => Action.hover(findSampleLine(lineNumber));

const expectFilePath = (filePath) =>
    Expectation.textEquals('.xl-react-widget-patch-sample .file-path', filePath);

const expectHeaderLabel = (headerLabel) =>
    Expectation.textEquals('.xl-react-widget-patch-sample .header-label', headerLabel);

const WidgetPatchSample = {
    expectLineContent,
    expectedNoSelectedLine,
    expectedSelectedLine,
    expectFilePath,
    expectHeaderLabel,
    findSampleLine,
    hoverLine,
    selectLine
};

export default WidgetPatchSample;
global.WidgetPatchSample = WidgetPatchSample;
