import {
    Action,
    ActionUtil,
    Expectation,
    WaitCondition
} from 'protractor-base-dsl';
import WidgetAutocomplete from '../widget/widget-autocomplete';

const suggestions = '.xl-react-widget-tags-with-autocomplete .testComponent ' +
    '.xl-react-autocomplete-suggestion-list .xl-react-autocomplete-suggestion';
const tags = '.xl-react-widget-tags-with-autocomplete .testComponent .react-tagsinput-tag';

const clickAddButton = () =>
    Action.click('.testComponent .react-tagsinput button');

const clickOnTitle = () =>
    Action.click('.xl-react-widget-tags-with-autocomplete .testComponent .xl-react-components-label');

const closeSuggestions = () => {
    const action = () => clickOnTitle();
    const condition = () =>
        WaitCondition.notDisplayed('.testComponent .xl-react-autocomplete-suggestion-list');
    ActionUtil.repeatAction(action, condition);
};

const clickOnContainer = () =>
    Action.click('.testComponent .react-tagsinput');

const clickOnPlaceholder = () =>
    Action.click('.testComponent .custom-placeholder');

const clickSelectAll = () =>
    Action.click('.testComponent .xl-react-autocomplete-suggestion-list-container .xl-react-link:first-child');

const clickClearAll = () =>
    Action.click('.testComponent .xl-react-autocomplete-suggestion-list-container .xl-react-link:last-child');

const expectAddButtonPresent = () =>
    Expectation.present('.testComponent .react-tagsinput button');

const expectAddButtonNotPresent = () =>
    Expectation.notPresent('.testComponent .react-tagsinput button');

const expectInputDisabled = () =>
    Expectation.disabled('.testComponent .react-tagsinput input');

const expectInputEnabled = () =>
    Expectation.enabled('.testComponent .react-tagsinput input');

const expectEmptyInputValue = () =>
    Expectation.attributeContainsValue('.testComponent .react-tagsinput input', 'value', '');

const expectTagPresent = (tagNumber, tagText) =>
    Expectation.present(`.testComponent .react-tagsinput-tag:nth-child(${tagNumber})[title="${tagText}"]`);

const expectTagNotPresent = (tagNumber) =>
    Expectation.notPresent(`.testComponent .react-tagsinput-tag:nth-child(${tagNumber})`);

const expectClearForTagPresent = (tagNumber) => {
    const tagSelector = `.testComponent .react-tagsinput-tag:nth-child(${tagNumber})`;
    Expectation.present(`${tagSelector} .react-tagsinput-remove`);
};

const expectClearForTagNotPresent = (tagNumber, tagText) => {
    const tagSelector = `.testComponent .react-tagsinput-tag:nth-child(${tagNumber})[title="${tagText}"]`;
    Expectation.notPresent(`${tagSelector} .react-tagsinput-remove`);
};

const expectCheckboxChecked = (optionNumber) => {
    const listSelector = '.testComponent .xl-react-autocomplete-suggestion-list';
    const suggestionOption = `${listSelector} .xl-react-autocomplete-suggestion:nth-child(${optionNumber}) .check-icon`;
    Expectation.present(suggestionOption);
};

const expectCheckboxUnChecked = (optionNumber) => {
    const listSelector = '.testComponent .xl-react-autocomplete-suggestion-list';
    const suggestionOption = `${listSelector} .xl-react-autocomplete-suggestion:nth-child(${optionNumber}) .check-icon`;
    Expectation.notPresent(suggestionOption);
};

const removeTag = (tagNumber) => {
    const tagSelector = `.testComponent .react-tagsinput-tag:nth-child(${tagNumber})`;
    Action.click(`${tagSelector} .react-tagsinput-remove`);
};

const setValue = (value) => {
    Action.typeText('.testComponent .react-tagsinput-input', value);
    browser.driver.sleep(500);
};

const selectSkin = (value) => {
    const groupSelector = '.xl-react-widget-tags-with-autocomplete .testBindings .radioButtonGroup';
    const skinSelector = `${groupSelector} input[name="skin"][value="${value}"]`;
    Action.click(skinSelector);
};

const openSuggestions = () => {
    const action = () => clickOnContainer();
    const condition = () => WaitCondition.displayed('.testComponent .xl-react-autocomplete-suggestion-list');

    ActionUtil.repeatAction(action, condition);
};

const selectCheckboxSkin = () => selectSkin('checkbox');

const expectCheckboxSkin = () => {
    openSuggestions();
    Expectation.present(`${suggestions}:nth-child(1) i.checkbox`);
};

const expectNoColorAndCheckboxSkin = () => {
    openSuggestions();
    Expectation.notPresent(`${suggestions}:nth-child(1) i.suggested-tag`);
    Expectation.notPresent(`${tags}:nth-child(1) i.option-icon`);
};

const selectColorAndCheckboxSkin = () => selectSkin('color-and-checkbox');

const addFreeTypedValue = (value) => {
    clickOnContainer();
    setValue(value);
    WidgetAutocomplete.clickEnterOnInput();
};

const expectColorAndCheckboxSkin = () => {
    openSuggestions();
    Expectation.present(`${suggestions}:nth-child(1) i.checkbox`);
    Expectation.present(`${suggestions}:nth-child(1) i.suggested-tag`);
    Expectation.present(`${tags}:nth-child(1) i.option-icon`);
};

const selectOptionAndExpectTag = (tagNumber, tagTitle) => {
    WidgetAutocomplete.expectSuggestionValue(tagNumber, tagTitle);
    WidgetAutocomplete.selectOption(tagNumber);
    expectTagPresent(tagNumber, tagTitle);
};

const WidgetTagsAutocomplete = {
    ...WidgetAutocomplete,
    addFreeTypedValue,
    closeSuggestions,
    clickAddButton,
    clickClearAll,
    clickOnContainer,
    clickOnPlaceholder,
    clickOnTitle,
    clickSelectAll,
    expectCheckboxChecked,
    expectCheckboxSkin,
    expectCheckboxUnChecked,
    expectClearForTagNotPresent,
    expectAddButtonNotPresent,
    expectClearForTagPresent,
    expectColorAndCheckboxSkin,
    expectEmptyInputValue,
    expectNoColorAndCheckboxSkin,
    expectAddButtonPresent,
    expectInputDisabled,
    expectInputEnabled,
    expectTagPresent,
    expectTagNotPresent,
    openSuggestions,
    removeTag,
    selectColorAndCheckboxSkin,
    selectCheckboxSkin,
    selectOptionAndExpectTag,
    selectSkin,
    setValue
};
export default WidgetTagsAutocomplete;
global.WidgetTagsAutocomplete = WidgetTagsAutocomplete;
