import DipString from '../../dsl/component/dip/dip-string';
import Form from '../../dsl/component/form/form';
import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';
import WidgetAutocomplete from '../../dsl/component/widget/widget-autocomplete';

describe('XL Dip Dynamic Form Empty Required Scenario', () => {
    beforeAll(() => {
        Navigation.openFormElement('dynamicFormEmpty');
        TestBindings.toggleRequired();
    });

    afterAll(() => {
        TestBindings.toggleRequired();
        TestBindings.resetPristinity();
    });

    it('should not validate disabled fields', () => {
        TestBindings.toggleDisabled();
        Form.expectAllFieldsAreValid();
        TestBindings.toggleDisabled();
    });

    it('should have all fields without value to present in model as invalid', () => {
        Form.expectAllNotEmptyFieldsAreInvalid();
    });

    it('should have correct descriptions', () => {
        Form.expectAllFieldsHaveDescription();
    });

    it('should show errors where field does not filled in values', () => {
        TestBindings.toggleShowMessages();
        Form.expectInvalidCategory(1);

        Form.expectErrorMessage('Label component String', 'This field is required');
        Form.expectErrorMessage('Label component StringLarge', 'This field is required');
        Form.expectErrorMessage('Label component StringMonaco', 'This field is required');
        Form.expectErrorMessage('Label component Integer', 'This field is required');
        Form.expectErrorMessage('Label component Set', 'This field is required');
        Form.expectErrorMessage('Label component List', 'This field is required');
        Form.expectErrorMessage('Label component CI', 'This field is required');
        Form.expectErrorMessage('Label String enum', 'This field is required');
        Form.expectErrorMessage('Label Radio enum', 'This field is required');
        Form.expectErrorMessage('Label component Map String String', 'This field is required');
        Form.expectErrorMessage('Label component Set Of CI', 'This field is required');
        Form.expectErrorMessage('Label component List Of CI', 'This field is required');
    });

    it('should not show errors anymore', () => {
        TestBindings.toggleShowMessages();
        Form.expectValidCategory(1);
        Form.expectAllFieldsHaveDescription();
    });

    it('should highlight group name when at least one field show error to the user', () => {
        Form.expectValidCategory(1);
        DipString.setValue('a');
        DipString.clearValue();
        Form.expectInvalidCategory(1);
    });

    it('should focus on an invalid field', () => {
        Form.setInputValue('Label component String', 'name');
        TestBindings.focus();
        WidgetAutocomplete.expectSuggestionListIsPresent();
        Form.cleanValue('Label component String');
    });
});
