import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetString from '../../../dsl/component/widget/widget-string';

describe('XL Widget String Medium Component Scenario', () => {
    beforeAll(() => {
        Navigation.openWidgetElement('widgetString1');
        WidgetString.toggleMediumSize();
    });

    afterAll(() => {
        TestBindings.resetPristinity();
        WidgetString.clearSelectedSize();
    });

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectValidToEqual('true');
        TestBindings.expectPristineToEqual('true');
        WidgetString.expectTextAreaPlaceholderToEqual('String placeholder');
        WidgetString.expectDescription('This is my string field.');
        WidgetString.expectLabel('String me');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        WidgetString.expectNotReadonlyTextArea();
        TestBindings.toggleDisabled();
        WidgetString.expectReadonlyTextArea();
        TestBindings.toggleDisabled();
        WidgetString.expectNotReadonlyTextArea();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        WidgetString.expectReadonlyTextArea();
        TestBindings.displayModeCreation();
        WidgetString.expectNotReadonlyTextArea();
        TestBindings.displayModeEdit();
        WidgetString.expectNotReadonlyTextArea();
    });

    it('should be editable on focus and clean value on repeated focus when password is toggled', () => {
        TestBindings.setInputType('password');
        WidgetString.setTextAreaValue('v');
        TestBindings.expectModelToEqual('v');
        WidgetString.hideChangedValue();
        TestBindings.expectModelToEqual('v');
        WidgetString.clickOnTextArea();
        WidgetString.expectTextAreaContentToEqual('');
        TestBindings.expectModelToEqual('v');
        WidgetString.hideChangedValue();
        TestBindings.expectModelToEqual('v');
        TestBindings.resetPristinity();
        WidgetString.expectTextAreaContentToEqual('*(unchanged) encrypted value*');
        TestBindings.setInputType('text');
        WidgetString.expectTextAreaContentToEqual('v');
        TestBindings.expectModelToEqual('v');
        WidgetString.clearValue();
    });

    it('should display cross if any text is typed in input field', () => {
        WidgetString.setTextAreaValue('1');
        TestBindings.expectModelToEqual('1');
        WidgetString.expectClearIconPresent();
        TestBindings.expectPristineToEqual('false');
        WidgetString.clearValue();
        TestBindings.expectPristineToEqual('false');
        WidgetString.expectClearIconNotPresent();
        TestBindings.expectModelToEqual('');
    });
});
