import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetString from '../../../dsl/component/widget/widget-string';
import MonacoEditor from '../../../dsl/common/monaco-editor';

describe('XL Widget String Monaco Editor Scenario', () => {
    beforeAll(() => {
        Navigation.openWidgetElement('widgetString1');
        WidgetString.toggleMonacoView();
    });

    afterAll(() => {
        TestBindings.resetPristinity();
        WidgetString.toggleDefaultView();
    });

    it('should change and apply default value', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectValidToEqual('true');
        TestBindings.expectPristineToEqual('true');
        WidgetString.expectFirstLineMonacoFieldHasText('');
        WidgetString.expectDescription('This is my string field.');
        WidgetString.expectLabel('String me');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        WidgetString.expectMonacoFieldNotReadonly();
        TestBindings.toggleDisabled();
        WidgetString.expectMonacoFieldReadonly();
        TestBindings.toggleDisabled();
        WidgetString.expectMonacoFieldNotReadonly();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        WidgetString.expectMonacoFieldReadonly();
        TestBindings.displayModeCreation();
        WidgetString.expectMonacoFieldNotReadonly();
        TestBindings.displayModeEdit();
        WidgetString.expectMonacoFieldNotReadonly();
    });

    it('should display cross if any text is typed in input field', () => {
        MonacoEditor.changeText('1');
        TestBindings.expectModelToEqual('1');
        WidgetString.expectClearIconPresent();
        TestBindings.expectPristineToEqual('false');
        WidgetString.clearValue();
        TestBindings.expectPristineToEqual('false');
        WidgetString.expectClearIconNotPresent();
        TestBindings.expectModelToEqual('');
    });
});
