import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetString from '../../../dsl/component/widget/widget-string';

describe('XL Widget String Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openWidgetElement('widgetString1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectValidToEqual('true');
        TestBindings.expectPristineToEqual('true');
        WidgetString.expectPlaceholderToEqual('String placeholder');
        WidgetString.expectDescription('This is my string field.');
        WidgetString.expectLabel('String me');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        WidgetString.expectNotReadonlyInput();
        TestBindings.toggleDisabled();
        WidgetString.expectReadonlyInput();
        TestBindings.toggleDisabled();
        WidgetString.expectNotReadonlyInput();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        WidgetString.expectReadonlyInput();
        TestBindings.displayModeCreation();
        WidgetString.expectNotReadonlyInput();
        TestBindings.displayModeEdit();
        WidgetString.expectNotReadonlyInput();
    });

    it('should display cross if any text is typed in input field', () => {
        WidgetString.setValue('1');
        TestBindings.expectModelToEqual(1);
        WidgetString.expectClearIconPresent();
        TestBindings.expectPristineToEqual('false');
        WidgetString.clearValue();
        TestBindings.expectPristineToEqual('false');
        WidgetString.expectClearIconNotPresent();
        TestBindings.expectModelToEqual('');
    });
});
