import Navigation from '../../dsl/common/navigation';
import TestBindings from '../../dsl/common/test-bindings';
import WidgetTime from '../../dsl/component/widget/widget-time';

describe('XL Widget Time Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openWidgetElement('widgetTime1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('November 9th 2016, 3:10:00 pm');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        WidgetTime.expectInputValue('3:10 PM');
        WidgetTime.expectDescription('Widget time description');
        WidgetTime.expectLabel('Choose time');
    });

    it('should allow to change time with the help of dialog', () => {
        WidgetTime.clickOnInput();
        WidgetTime.expectTimePickerPresent();
        WidgetTime.expectHour('3');
        WidgetTime.expectMinute('10');
        WidgetTime.expectPm();
        WidgetTime.plusHour();
        TestBindings.expectPristineToEqual('false');
        WidgetTime.expectInputValue('4:10 PM');
        WidgetTime.expectHour('4');
        WidgetTime.expectMinute('10');

        WidgetTime.minusHour();
        WidgetTime.minusHour();
        WidgetTime.expectInputValue('2:10 PM');
        WidgetTime.expectHour('2');
        WidgetTime.expectMinute('10');

        WidgetTime.plusMinute();
        TestBindings.expectPristineToEqual('false');
        WidgetTime.expectInputValue('2:11 PM');
        WidgetTime.expectHour('2');
        WidgetTime.expectMinute('11');

        WidgetTime.minusMinute();
        WidgetTime.minusMinute();
        TestBindings.expectPristineToEqual('false');
        WidgetTime.expectInputValue('2:09 PM');
        WidgetTime.expectHour('2');
        WidgetTime.expectMinute('09');
        WidgetTime.clickOnLabel();
        WidgetTime.expectTimePickerNotPresent();
    });

    it('should not open "select date" dialog when disabled', () => {
        WidgetTime.expectInputEnabled();
        WidgetTime.expectTimePickerNotPresent();
        TestBindings.toggleDisabled();
        WidgetTime.expectInputDisabled();
        WidgetTime.clickOnInput();
        WidgetTime.expectTimePickerNotPresent();
        TestBindings.toggleDisabled();
        WidgetTime.expectInputEnabled();
    });
});
