import {delayedHandler, labelToClassName} from '~/app/xl-react-buttons/xl-react-button-utils';

describe('xl-react-button-utils', () => {
    jest.useFakeTimers();

    const setState = jest.fn((state, callback) => callback());
    const onClick = jest.fn();

    beforeEach(() => {
        setState.mockClear();
        onClick.mockClear();
    });

    const event = {};

    describe('delayedHandler', () => {
        it('should invoke handler if component is clickable', () => {
            const component = {
                inactiveTime: 500,
                props: {onClick},
                setState,
                state: {clickable: true},
                timeouts: []
            };

            delayedHandler(component, 'onClick', event);

            expect(component.setState).toBeCalled();
            expect(component.timeouts.length).toBe(1);
            expect(component.props.onClick).toBeCalledWith(event);
        });

        it('should not invoke handler if component is not clickable', () => {
            const component = {
                inactiveTime: 500,
                props: {onClick},
                setState,
                state: {clickable: false},
                timeouts: []
            };

            delayedHandler(component, 'onClick', event);

            expect(component.setState).not.toBeCalled();
            expect(component.timeouts.length).toBe(0);
            expect(component.props.onClick).not.toBeCalled();
        });
    });

    describe('labelToClassName', () => {
        it('should convert string label to class name', () => {
            expect(labelToClassName('Deploy')).toEqual('deploy');
            expect(labelToClassName('Modify Plan')).toEqual('modify-plan');
            expect(labelToClassName(1)).toEqual('');
            expect(labelToClassName({})).toEqual('');
        });
    });
});
