import * as service from '~/app/xl-react-dip/input/input-service';

describe('xl-react-dip-map-string-string-service', () => {
    describe('blurFieldOnSubmit', () => {
        it('should blur the field if RETURN key pressed', () => {
            const inputField1 = {blur: jest.fn()};
            const event1 = {keyCode: 13};
            service.blurFieldOnSubmit(inputField1, event1);
            expect(inputField1.blur).toBeCalled();

            const inputField2 = {blur: jest.fn()};
            const event2 = {keyCode: 10};
            service.blurFieldOnSubmit(inputField2, event2);
            expect(inputField2.blur).not.toBeCalled();
        });
    });
    describe('computeNewStateOnNewKeyBlur', () => {
        it('should compute new state based on current state', () => {
            const currentState1 = {
                model: {},
                newKey: 'Key1   ',
                newValueFocused: false
            };
            expect(service.computeNewStateOnNewKeyBlur(currentState1)).toEqual({newKey: 'Key1'});

            const currentState2 = {
                newKey: ' ',
                newValueFocused: true
            };
            expect(service.computeNewStateOnNewKeyBlur(currentState2, {})).toEqual(
                {emptyKey: true, hasNewKeyError: true, model: {}}
            );

            const currentState3 = {
                newKey: '  akey  ',
                newValueFocused: true
            };
            expect(service.computeNewStateOnNewKeyBlur(currentState3, {akey: 'aValue'})).toEqual(
                {duplicateKey: 'akey', hasNewKeyError: true, model: {akey: 'aValue'}}
            );
        });
    });
    describe('isKeyUnique', () => {
        const objectModel = {cat: 1, donkey: 3, pony: 2};
        const stringModel = ['cat', 'pony', 'donkey'];

        it('should return true if key is not in the object model', () => {
            expect(service.isKeyUnique('dog')(objectModel)).toBeTruthy();
            expect(service.isKeyUnique('cat', 'cat')(objectModel)).toBeTruthy();
        });
        it('should return false if key is not in the object model', () => {
            expect(service.isKeyUnique('cat')(objectModel)).toBeFalsy();
            expect(service.isKeyUnique('cat', 'donkey')(objectModel)).toBeFalsy();
        });

        it('should return true if key is not in the string model', () => {
            expect(service.isKeyUnique('dog')(stringModel)).toBeTruthy();
            expect(service.isKeyUnique('cat', 'cat')(stringModel)).toBeTruthy();
        });

        it('should return false if key is not in the string model', () => {
            expect(service.isKeyUnique('cat')(stringModel)).toBeFalsy();
            expect(service.isKeyUnique('cat', 'donkey')(stringModel)).toBeFalsy();
        });
    });
});
