import React from 'react';
import {mount} from 'enzyme';

import {XlReactWidgetFilter} from '~/app/xl-react-widget/xl-react-widget-filter/xl-react-widget-filter';

describe('XlReactWidgetFilter', () => {
    it('should render widget without label', () => {
        const wrapper = mount(<XlReactWidgetFilter/>);
        expect(wrapper.find('.xl-react-components-input-full')).toExist();
    });

    it('should render widget with the label', () => {
        const wrapper = mount(<XlReactWidgetFilter metadata={{label: 'I\'m filter'}}/>);
        const label = wrapper.find('XlReactWidgetLabel');
        expect(label).toExist();
        expect(label.prop('metadata').label).toEqual('I\'m filter');
        expect(wrapper.find('.xl-react-components-input')).toExist();
        expect(wrapper.find('.xl-react-components-input-full')).not.toExist();
        expect(wrapper.find('.close-icon')).not.toExist();
    });

    it('should render widget without placeholder', () => {
        const wrapper = mount(<XlReactWidgetFilter/>);
        const input = wrapper.find('input');
        expect(input.props().placeholder).toBeUndefined();
    });

    it('should render widget with placeholder', () => {
        const wrapper = mount(<XlReactWidgetFilter metadata={{placeholder: 'placeholder test'}}/>);
        const input = wrapper.find('input');
        expect(input.props().placeholder).toEqual('placeholder test');
    });

    it('should properly render model and show "clear" button', () => {
        const wrapper = mount(<XlReactWidgetFilter model={'criteria'}/>);
        expect(wrapper.find('.xl-react-components-input-full')).toExist();
        expect(wrapper.find('.close-icon')).toExist();
    });

    it('should propagate changes back with the help of callback function', () => {
        const onUpdateFn = jest.fn();
        const wrapper = mount(<XlReactWidgetFilter onUpdate={onUpdateFn}/>);
        const input = () => wrapper.find('.xl-react-components-input-full input');
        input().simulate('change', {target: {value: 'My new value'}});

        expect(onUpdateFn.mock.calls).toEqual([['My new value']]);
        expect(input()).toHaveValue('My new value');

        wrapper.find('.close-icon').simulate('click');
        expect(onUpdateFn.mock.calls).toEqual([['My new value'], ['']]);
        expect(input()).toHaveValue('');
    });
});
