import React from 'react';
import {mount, ReactWrapper} from 'enzyme';
import {Modal} from 'react-bootstrap';
import ReactTestUtils from 'react-dom/test-utils';
import {XlReactWidgetModalDialog} from '~/app/xl-react-widget/xl-react-widget-modal-dialog/xl-react-widget-modal-dialog'; // eslint-disable-line max-len

describe('XlReactWidgetModalDialog', () => {
    it('by default modal dialog shouldn\'t be visible', () => {
        const wrapper = mount(<XlReactWidgetModalDialog showModal={false}/>);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        expect(dialog).toBeNull();
    });

    it('should be possible to make it visible', () => {
        const wrapper = mount(<XlReactWidgetModalDialog showModal={true}/>);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        const modal = new ReactWrapper(dialog, dialog);

        expect(modal.find('.modal-dialog')).toBeDefined();
        expect(modal.find('.modal-content')).toBeDefined();
        expect(modal.find('.modal-header')).toBeDefined();
    });

    it('should provide cross for the user to close modal dialog', () => {
        const handleCloseFn = jest.fn();

        const wrapper = mount(<XlReactWidgetModalDialog handleClose={handleCloseFn} showModal={true}/>);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();

        const closeBtn = dialog.getElementsByClassName('close-icon')[0];

        ReactTestUtils.Simulate.click(closeBtn);
        expect(handleCloseFn).toHaveBeenCalledTimes(1);
    });

    it('should render child component inside modal dialog', () => {
        const wrapper = mount(
            <XlReactWidgetModalDialog showModal={true}>
                <span className="my-element"/>
            </XlReactWidgetModalDialog>
        );
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        const modal = new ReactWrapper(dialog, dialog);
        expect(modal.find('.my-element')).toBeDefined();
    });

    it('should be possible to show title', () => {
        const wrapper = mount(<XlReactWidgetModalDialog showModal={true} title="My Modal Title"/>);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        const titleElement = dialog.getElementsByClassName('title')[0];
        expect(titleElement.textContent).toBe('My Modal Title');
    });
});
