import {testSaga} from 'redux-saga-test-plan';
import {call} from 'redux-saga/effects';
import {
    sort,
    sortSaga
} from '~/app/xl-react-widget/xl-react-widget-redux-table/sagas/xl-react-widget-redux-table-sorting-saga';
import {getSortingWithSortKey} from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-table-selectors';
import {
    fetchData,
    withSpinner
} from '~/app/xl-react-widget/xl-react-widget-redux-table/sagas/xl-react-widget-redux-table-common-saga';
import tableActions
    from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-redux-table-actions';

describe('Widget Redux Table Sort Saga', () => {
    const mockApi = jest.fn();
    const name = 'firstname';
    const order = 'asc';
    const api = mockApi;
    const tableId = 'table-1';
    const prevSort = {name: 'firstColumn', order: 'asc'};

    describe('sort', () => {
        it('should call pagination api with a spinner during waiting a call response', () => {
            const saga = testSaga(sort, {api, name, order, tableId});
            saga.next().select(getSortingWithSortKey, tableId);
            saga.next(prevSort).put(tableActions.setSorting(tableId, name, order));
            saga.next().call(
                fetchData,
                tableId,
                api,
                {change: {previous: prevSort, type: 'sorting'}}
            );
            saga.next().isDone();
        });
    });

    describe('sortSaga', () => {
        it('should set pagination settings and fetch data based on this', () => {
            const saga = testSaga(sortSaga, {api, name, order, tableId});
            saga.next().call(withSpinner, tableId, call(sort, {api, name, order, tableId}));
            saga.next().isDone();
        });
    });
});
