import React from 'react';
import {shallow} from 'enzyme';

import {XlReactWidgetSpinner} from '~/app/xl-react-widget/xl-react-widget-spinner/xl-react-widget-spinner';

describe('Spinner component', () => {
    const wrapper = shallow(<XlReactWidgetSpinner show={false}/>);

    it('should return null when not shown', () => {
        expect(wrapper.html()).toBeNull();
    });

    it('should show component when shown as overlay', () => {
        wrapper.setProps({show: true});
        expect(wrapper.find('div').length).toBe(2);
        expect(wrapper.find('div').at(0).hasClass('xl-react-widget-spinner-container')).toBe(true);
        expect(wrapper.find('div').at(0).hasClass('xl-react-widget-spinner-overlay')).toBe(true);
        expect(wrapper.find('div').at(1).hasClass('xl-react-widget-spinner')).toBe(true);
    });

    it('should show component when shown as block', () => {
        wrapper.setProps({overlay: false, show: true});
        expect(wrapper.find('div').length).toBe(2);
        expect(wrapper.find('div').at(0).hasClass('xl-react-widget-spinner-container')).toBe(true);
        expect(wrapper.find('div').at(0).hasClass('xl-react-widget-spinner-block')).toBe(true);
        expect(wrapper.find('div').at(1).hasClass('xl-react-widget-spinner')).toBe(true);
    });

    it('should show component when shown as block with message', () => {
        wrapper.setProps({message: 'Message', overlay: false, show: true});
        expect(wrapper.find('div').length).toBe(3);
        expect(wrapper.find('div').at(0).hasClass('xl-react-widget-spinner-container')).toBe(true);
        expect(wrapper.find('div').at(0).hasClass('xl-react-widget-spinner-block')).toBe(true);
        expect(wrapper.find('div').at(1).hasClass('xl-react-widget-spinner')).toBe(true);
        expect(wrapper.find('div').at(2).text()).toBe('Message');
    });
});
