import R from 'ramda';
import $ from 'jquery';
import {
    getNoOfTagsToDisplay,
    getRemainingTags
} from
    '~/app/xl-react-widget/xl-react-widget-tags/xl-react-widget-tags-dom-manipulation';

const model = R.map(
    (value) => ({label: value, value}),
    ['App1', 'App2', 'App3', 'App4', 'App5', 'App6']
);

const tagsField = $('<div style="width: 100px;"></div>');

describe('xl-react-widget-tags-dom-manipulation', () => {
    describe('getNoOfTagsToDisplay', () => {
        it('should get the number of tags to display', () => {
            expect(getNoOfTagsToDisplay(100, model)).toEqual(6);
            expect(getNoOfTagsToDisplay(200, model)).toEqual(1);
            expect(getNoOfTagsToDisplay(300, model)).toEqual(2);
            expect(getNoOfTagsToDisplay(1000, model)).toEqual(11);
            expect(getNoOfTagsToDisplay(0, model)).toEqual(6);
        });
    });

    describe('getRemainingTags', () => {
        it('should get the remaining tags other than the displayed ones', () => {
            expect(getRemainingTags(tagsField, model)).toEqual([]);

            tagsField.width(200);
            expect(getRemainingTags(tagsField, model)).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App2', 'App3', 'App4', 'App5', 'App6']
            ));

            tagsField.width(300);
            expect(getRemainingTags(tagsField, model)).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App3', 'App4', 'App5', 'App6']
            ));

            tagsField.width(1000);
            expect(getRemainingTags(tagsField, model)).toEqual([]);

            tagsField.width(0);
            expect(getRemainingTags(tagsField, model)).toEqual([]);
        });
    });
});